# encoding: utf-8
"""
@project: djangoModel->user_apply_apis
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户申请APIS
@created_time: 2023/9/7 13:54
"""
from rest_framework.decorators import api_view
from rest_framework.views import APIView

from ..services.user_apply_services import UserApplyServices
from ..utils.custom_response import util_response
from ..utils.custom_tool import request_params_wrapper
from ..utils.user_wrapper import user_authentication_force_wrapper, user_authentication_wrapper


class UserApplyApis(APIView):

    @api_view(["GET"])
    @request_params_wrapper
    @user_authentication_wrapper
    def apply_type_list(self, *args, request_params, user_info, **kwargs):
        """审批类型列表"""
        data, err = UserApplyServices.apply_type_list()
        return util_response(data=data)

    @api_view(["GET", "POST", "PUT"])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def add_user_apply(self, *args, request_params, user_info, **kwargs):
        request_params.setdefault("user_id", user_info.get("user_id"))
        data, err = UserApplyServices.add_apply_record(request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET", "POST", "PUT"])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def agree_user_apply(self, *args, request_params, user_info, **kwargs):
        request_params.setdefault("verify_user_id", user_info.get("user_id"))
        request_params.setdefault("result", "VERIFY_PASS")
        pk = request_params.get("pk")
        data, err = UserApplyServices.edit_apply_record(request_params, pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET", "POST", "PUT"])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def disagree_user_apply(self, *args, request_params, user_info, **kwargs):
        pk = request_params.get("pk")
        request_params.setdefault("verify_user_id", user_info.get("user_id"))
        request_params.setdefault("result", "VERIFY_REJECT")
        reject_reason = request_params.get("reject_reason")
        if not reject_reason:
            return util_response(err=1000, msg="决绝理由不能为空")

        data, err = UserApplyServices.edit_apply_record(request_params, pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET", "POST", "PUT"])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def user_apply_list(self, *args, request_params, user_info, **kwargs):
        request_params.setdefault("user_id", user_info.get("user_id"))
        data, err = UserApplyServices.apply_record_list(request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    @api_view(["GET", "POST", "PUT"])
    @request_params_wrapper
    @user_authentication_force_wrapper
    def user_apply_statistics(self, *args, request_params, user_info, **kwargs):
        request_params.setdefault("user_id", user_info.get("user_id"))
        data, err = UserApplyServices.apply_record_agg(request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



# Generated by Django 4.2.17 on 2025-08-23 17:41

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import xj_user.utils.j_ulid_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='BaseInfo',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', xj_user.utils.j_ulid_field.JULIDField(blank=True, db_index=True, default='1k3b4v3jv09y', max_length=32, null=True, unique=True, verbose_name='UUID')),
                ('user_no', models.CharField(blank=True, db_index=True, max_length=32, null=True, unique=True, verbose_name='用户编号')),
                ('username', models.CharField(blank=True, db_index=True, help_text='登录的用户名', max_length=32, null=True, unique=True, verbose_name='用户名')),
                ('nickname', models.CharField(blank=True, max_length=128, null=True, verbose_name='昵称')),
                ('fullname', models.CharField(blank=True, db_index=True, help_text='用户的姓名（未实名验证）', max_length=128, null=True, verbose_name='姓名')),
                ('phone', models.CharField(blank=True, db_index=True, help_text='登录绑定手机号', max_length=16, null=True, verbose_name='手机')),
                ('email', models.EmailField(blank=True, db_index=True, help_text='登录绑定邮箱', max_length=254, null=True, verbose_name='邮箱')),
                ('avatar', models.CharField(blank=True, max_length=255, null=True, verbose_name='个人头像')),
                ('user_type', models.CharField(blank=True, choices=[('PERSON', '个人用户'), ('COMPANY', '企业用户'), ('COMPANYADMIN', '企业管理员'), ('ADMIN', '管理员用户')], db_index=True, default='PERSON', help_text='PERSON 个人用户, COMPANY 企业用户, ADMIN 管理员用户', max_length=32, null=True, verbose_name='用户类型')),
                ('user_vip', models.IntegerField(blank=True, choices=[(1, '普通用户'), (2, '高级会员'), (3, '超级会员'), (4, '年费会员')], db_index=True, default=1, help_text='用户会员权限', verbose_name='用户会员')),
                ('user_level', models.IntegerField(blank=True, db_index=True, default=1, verbose_name='用户等级')),
                ('user_info', models.JSONField(blank=True, help_text='用户扩展信息', null=True, verbose_name='用户信息')),
                ('register_time', models.DateTimeField(blank=True, default=django.utils.timezone.now, null=True, verbose_name='注册时间')),
                ('register_ip', models.GenericIPAddressField(blank=True, null=True, verbose_name='注册IP')),
                ('privacies', models.JSONField(blank=True, help_text='用户隐私设置', null=True, verbose_name='用户隐私设置')),
                ('is_auth', models.BooleanField(blank=True, default=False, null=True, verbose_name='是否实名认证')),
                ('is_delete', models.BooleanField(blank=True, default=False, null=True, verbose_name='是否删除')),
                ('is_using', models.BooleanField(blank=True, default=True, null=True, verbose_name='是否使用')),
            ],
            options={
                'verbose_name_plural': '01. 用户 - 基础信息',
                'db_table': 'user_base_info',
            },
        ),
        migrations.CreateModel(
            name='ExtendField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('field_index', models.CharField(blank=True, choices=[('field_1', '自定义字段1'), ('field_2', '自定义字段2'), ('field_3', '自定义字段3'), ('field_4', '自定义字段4'), ('field_5', '自定义字段5'), ('field_6', '自定义字段6'), ('field_7', '自定义字段7'), ('field_8', '自定义字段8'), ('field_9', '自定义字段9'), ('field_10', '自定义字段10'), ('field_11', '自定义字段11'), ('field_12', '自定义字段12'), ('field_13', '自定义字段13'), ('field_14', '自定义字段14'), ('field_15', '自定义字段15')], help_text='映射到扩展数据表的字段名，如：field_x', max_length=8, null=True, unique=True, verbose_name='映射索引名')),
                ('field', models.CharField(blank=True, help_text='当已有字段不能满足的时候的扩展字段', max_length=32, null=True, unique=True, verbose_name='自定义字段')),
                ('label', models.CharField(blank=True, help_text='自定义字段的标签，用于前端显示', max_length=32, null=True, unique=True, verbose_name='标签')),
                ('type', models.CharField(blank=True, choices=[('bool', '布尔型-bool'), ('int', '整型-int'), ('float', '浮点型-float'), ('number', '数字类型-number'), ('string', '字符串型-string'), ('text', '长文本型-text'), ('plain', '普通文字-plain'), ('input', '输入框-plain'), ('password', '密码框-password'), ('textarea', '多行文本框-textarea'), ('editor', '富文本编辑器-editor'), ('switch', '开关切换-switch'), ('select', '选择框-select'), ('radio', '单选框-radio'), ('checkbox', '多选框-checkbox'), ('cascader', '级联选择器-cascader'), ('color', '色彩选择器-color'), ('slot', '插槽-slot'), ('image', '图片-image'), ('audio', '音频-audio'), ('video', '视频-video'), ('file', '文件-file'), ('upload', '上传类型-upload'), ('time', ' 时间-time'), ('datetime', '日期时间-datetime'), ('date', '日期-date'), ('month', '月份-month'), ('year', '年-year')], help_text='自定义字段的数据类型，必须和前端相匹配', max_length=32, null=True, verbose_name='类型')),
                ('unit', models.CharField(blank=True, help_text='自定义字段的显示单位，空值表示不需要配置单位', max_length=32, null=True, verbose_name='单位')),
                ('default', models.CharField(blank=True, help_text='自定义字段的默认值，当前端用户不配置字段值时，使用该默认值', max_length=255, null=True, verbose_name='默认值')),
                ('config', models.JSONField(blank=True, help_text='用于前端自由配置一些属性，如选择列表', null=True, verbose_name='配置')),
                ('sort', models.IntegerField(blank=True, null=True, verbose_name='排序')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段描述')),
            ],
            options={
                'verbose_name_plural': '04. 用户 - 扩展字段',
                'db_table': 'user_extend_field',
            },
        ),
        migrations.CreateModel(
            name='Platform',
            fields=[
                ('id', models.IntegerField(help_text='必填。自动生成。', primary_key=True, serialize=False, verbose_name='ID')),
                ('platform_id', models.IntegerField(help_text='必填。不自动生成，由运营人员统一设置。', unique=True, verbose_name='平台ID')),
                ('platform_code', models.CharField(help_text='必填。平台唯一识别码', max_length=16, unique=True, verbose_name='平台代码')),
                ('platform_name', models.CharField(help_text='必填。平台名称可以是中文、英文、俄文等。', max_length=32, verbose_name='平台名称')),
                ('root_category_value', models.CharField(help_text='唯一值，信息模块根类别值', max_length=32, verbose_name='根类别值')),
            ],
            options={
                'verbose_name_plural': '07. 用户 - 平台表',
                'db_table': 'user_platform',
            },
        ),
        migrations.CreateModel(
            name='UserSsoServe',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sso_code', models.CharField(blank=True, max_length=255, null=True, verbose_name='单点登录代码')),
                ('sso_name', models.CharField(blank=True, max_length=255, null=True, verbose_name='单点登录名')),
                ('sso_url', models.CharField(blank=True, max_length=255, null=True, verbose_name='单点登录地址')),
                ('sso_appid', models.CharField(blank=True, max_length=255, null=True, verbose_name='AppID')),
                ('description', models.CharField(blank=True, max_length=255, null=True, verbose_name='描述')),
                ('sso_account', models.ForeignKey(db_column='sso_account_id', db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.baseinfo', verbose_name='单点登录绑定账户')),
            ],
            options={
                'verbose_name_plural': '11. 用户 - 单点登录服务表',
                'db_table': 'user_sso_serve',
            },
        ),
        migrations.CreateModel(
            name='UserSsoToUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sso_unicode', models.CharField(blank=True, help_text='？？？', max_length=255, null=True, verbose_name='单点登录地址')),
                ('sso_ticket', models.CharField(blank=True, max_length=255, null=True, verbose_name='单点登录票据')),
                ('union_code', models.CharField(blank=True, max_length=255, null=True, verbose_name='微信唯一ID')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('is_delete', models.BooleanField(default=False, verbose_name='是否删除')),
                ('sso_serve', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.userssoserve', verbose_name='单点登录服务ID')),
                ('user', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.baseinfo', verbose_name='用户')),
            ],
            options={
                'verbose_name_plural': '12. 用户 - 多对多单点登录记录表',
                'db_table': 'user_sso_to_user',
            },
        ),
        migrations.CreateModel(
            name='UserBankCards',
            fields=[
                ('id', models.AutoField(primary_key=True, serialize=False, verbose_name='ID')),
                ('bank_card_num', models.CharField(blank=True, max_length=32, null=True, verbose_name='银行卡号')),
                ('open_account_bank', models.CharField(blank=True, max_length=32, null=True, verbose_name='开户银行')),
                ('opening_branch', models.CharField(blank=True, max_length=128, null=True, verbose_name='开户支行')),
                ('is_default', models.BooleanField(blank=True, null=True, verbose_name='是否默认卡')),
                ('ext', models.JSONField(blank=True, null=True, verbose_name='扩展数据')),
                ('sort', models.IntegerField(blank=True, null=True, verbose_name='排序')),
                ('remark', models.CharField(blank=True, max_length=128, null=True, verbose_name='备注')),
                ('user', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.baseinfo', verbose_name='开户人')),
            ],
            options={
                'verbose_name_plural': '15. 用户 - 用户银行卡信息',
                'db_table': 'user_bank_cards',
            },
        ),
        migrations.CreateModel(
            name='PlatformsToUsers',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('platform_user_id', models.BigIntegerField(blank=True, db_index=True, null=True, verbose_name='平台用户ID')),
                ('platform', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.platform', to_field='platform_id', verbose_name='平台')),
                ('user', models.ForeignKey(db_column='user_id', on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.baseinfo', verbose_name='用户')),
            ],
            options={
                'verbose_name_plural': '09. 用户 - 多对多平台记录表',
                'db_table': 'user_platforms_to_users',
                'unique_together': {('user', 'platform')},
            },
        ),
        migrations.CreateModel(
            name='DetailInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('real_name', models.CharField(blank=True, db_index=True, help_text='实名认证后的真名', max_length=50, null=True, verbose_name='实名姓名')),
                ('sex', models.CharField(blank=True, choices=[('0', '未知'), ('1', '男'), ('2', '女')], help_text='0未知 1男 2女', max_length=1, null=True, verbose_name='性别')),
                ('birth', models.DateField(blank=True, null=True, verbose_name='生日')),
                ('tags', models.CharField(blank=True, help_text='多个标签以英文;号为分隔符', max_length=255, null=True, verbose_name='个人标签')),
                ('signature', models.CharField(blank=True, max_length=255, null=True, verbose_name='个性签名')),
                ('avatar', models.CharField(blank=True, max_length=2000, null=True, verbose_name='个人头像')),
                ('cover', models.CharField(blank=True, max_length=2000, null=True, verbose_name='个人主页封面')),
                ('id_card_type', models.CharField(blank=True, choices=[('IDENTITY', '身份证'), ('PASSPORT', '护照'), ('DRIVING', '驾驶证'), ('BUSINESS', '营业执照')], help_text='用于实名认证姓名时的证件类型', max_length=32, null=True, verbose_name='证件类型')),
                ('id_card_no', models.CharField(blank=True, max_length=32, null=True, verbose_name='证件号码')),
                ('language', models.CharField(blank=True, choices=[('cn', '简体中文'), ('en', 'English'), ('ru', 'Русский'), ('tc', '繁体中文'), ('fr', 'Français'), ('de', 'Deutsch'), ('jp', '日本語')], max_length=2, null=True, verbose_name='我使用的语言')),
                ('region_code', models.BigIntegerField(blank=True, null=True, verbose_name='所在地行政区划编码')),
                ('more', models.JSONField(blank=True, help_text='更多信息用来存放用户可能填写的扩展内容，由于很多信息不是必填或必须存在的，因此不单独建字段。', null=True, verbose_name='更多信息')),
                ('field_1', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段1')),
                ('field_2', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段2')),
                ('field_3', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段3')),
                ('field_4', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段4')),
                ('field_5', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段5')),
                ('field_6', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段6')),
                ('field_7', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段7')),
                ('field_8', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段8')),
                ('field_9', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段9')),
                ('field_10', models.CharField(blank=True, max_length=255, null=True, verbose_name='字段10')),
                ('field_11', models.TextField(blank=True, null=True, verbose_name='字段11')),
                ('field_12', models.TextField(blank=True, null=True, verbose_name='字段12')),
                ('field_13', models.TextField(blank=True, null=True, verbose_name='字段13')),
                ('field_14', models.TextField(blank=True, null=True, verbose_name='字段14')),
                ('field_15', models.TextField(blank=True, null=True, verbose_name='字段15')),
                ('user', models.OneToOneField(db_column='user_id', db_constraint=False, help_text='登录用的用户名', on_delete=django.db.models.deletion.DO_NOTHING, related_name='user_detail_info', to='xj_user.baseinfo', verbose_name='用户')),
            ],
            options={
                'verbose_name_plural': '02. 用户 - 详细信息',
                'db_table': 'user_detail_info',
            },
        ),
        migrations.AddField(
            model_name='baseinfo',
            name='platforms_to_users',
            field=models.ManyToManyField(through='xj_user.PlatformsToUsers', to='xj_user.platform'),
        ),
        migrations.CreateModel(
            name='Auth',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(blank=True, help_text='登录的密码', max_length=255, null=True, verbose_name='密码')),
                ('plaintext', models.CharField(blank=True, max_length=255, null=True, verbose_name='PT')),
                ('salt', models.CharField(blank=True, help_text='加密盐由六位数字和字母组成，区分大小写', max_length=32, null=True, verbose_name='盐')),
                ('algorithm', models.CharField(blank=True, choices=[('DEFAULT', 'Default'), ('DISCUZ', 'Discuz (md5(md5(pwd).salt))'), ('SPSS', 'Spss (sha1(rsa(cip).salt))')], help_text='系统同时支持多种加密算法，旧版为MD5，新版为SHA1', max_length=16, null=True, verbose_name='加密算法')),
                ('token', models.CharField(blank=True, help_text='用户令牌，用于验证用户有效期', max_length=255, null=True, verbose_name='临时令牌')),
                ('ticket', models.CharField(blank=True, help_text='用户提供第三方单点登录的票据', max_length=255, null=True, verbose_name='临时票据')),
                ('last_update_ip', models.GenericIPAddressField(blank=True, null=True, verbose_name='最后登录网络地址')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('update_time', models.DateTimeField(auto_now=True, null=True, verbose_name='最后登录时间')),
                ('platform', models.ForeignKey(blank=True, db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.platform', verbose_name='平台')),
                ('user', models.ForeignKey(db_constraint=False, on_delete=django.db.models.deletion.DO_NOTHING, to='xj_user.baseinfo', verbose_name='用户')),
            ],
            options={
                'verbose_name_plural': '03. 用户 - 安全认证',
                'db_table': 'user_auth',
            },
        ),
    ]



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: CURD 工具
@created_time: 2022/6/15 14:14
"""

# 请求参数解析
import json

import typing


def x_get_params(request, content_type=None, **kwargs):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    query_param = request.GET or {}  # 在APIView才有request.query_params方法
    body_param = request.POST or {}
    try:
        json_param = json.loads(request.body)
    except Exception as e:
        json_param = {}
    result_param = {}
    result_param.update(query_param)
    result_param.update(body_param)
    if isinstance(json_param, dict):
        result_param.update(json_param)
    return result_param



# encoding: utf-8
"""
@project: djangoModel->sync_table_structure
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 更新表结构
@created_time: 2023/8/22 9:09
"""
from django.core.management import BaseCommand


class Command(BaseCommand):
    # 帮助文本, 一般备注命令的用途及如何使用。
    help = "测试调试结构"

    # 给命令添加一个名为name的参数
    def add_arguments(self, parser):
        parser.add_argument(
            "--delete",
            action="store_true",
            help="是否执行删除测试用例",
        )
        parser.add_argument(
            "--add",
            action="store_true",
            help="是否执行添加测试用例",
        )
        parser.add_argument(
            "--edit",
            action="store_true",
            help="是否执行编辑测试用例",
        )
        parser.add_argument(
            "--list",
            action="store_true",
            help="是否执行列表测试用例",
        )

    # 核心业务逻辑，通过options字典接收name参数值，拼接字符串后输出
    def handle(self, *args, **options):
        from xj_user.services.user_apply_services import UserApplyServices
        if options.get("add"):
            data, err = UserApplyServices.add_apply_record(params={
                "apply_type_id": 1,
                "user_id": 116,
                "remark": "加急备注",
                "verify_files": {"id_card_front": "https://baidu.com"}
            })
            print(data, err)
        if options.get("edit"):
            data, err = UserApplyServices.edit_apply_record(
                params={
                    "apply_type_id": 1,
                    "user_id": 1,
                    "remark": "加急备注",
                    "verify_files": {"id_card_front": "https://baidu.com"},
                    "reject_reason": "格式不合格,重新制作",
                    "verify_user_id": 298,
                    # "result": UserApplyServices.VERIFY_PASS
                    "result": UserApplyServices.VERIFY_REJECT

                },
                pk=6
            )
            print(data, err)

        if options.get("list"):
            data, err = UserApplyServices.apply_record_list()
            print(data, err)



"""
Created on 2022-04-12
@author:刘飞
@description:自定义异常捕获
"""
from rest_framework.views import exception_handler
from django.http import JsonResponse


def custom_exception_handler(exc, context):
    """
    自定义异常返回
    """

    response = exception_handler(exc, context)

    def get_data():
        res_dic = ''
        for k in list(response.data.keys()):
            if k != 'detail':
                res_dic = str(k) + ':' + str(response.data[k])
                response.data.pop(k, None)
        return res_dic

    if response is not None:
        if isinstance(response.data, list):  # 兼容serializers
            res = {
                "msg": response.data[0],
                "err": response.status_code,
                "data": {}
            }
            return JsonResponse(res)
        elif isinstance(response.data, dict):  # 兼容serializers, 抛出的ValidationError
            msg = ''
            for _k, _v in zip(response.data.keys(), response.data.values()):
                msg += f"{_k}: {_v[0] if isinstance(_v, list) else _v}"
            res = {
                "msg": msg,
                "err": response.status_code,
                "data": {}
            }
            return JsonResponse(res)
        else:
            response.data['msg'] = response.data.pop('detail') if response.data.detail('detail') else get_data()
            response.data['err'] = response.status_code
            response.data['data'] = {}

    return response



# _*_coding:utf-8_*_
from rest_framework.views import APIView

from ..services.user_bank_service import UserBankCardsService
from ..services.user_service import UserService
from ..utils.custom_tool import request_params_wrapper, format_params_handle
from ..utils.model_handle import util_response
from ..utils.user_wrapper import user_authentication_force_wrapper
# from ..utils.utility_method import extract_values, replace_key_in_dict_replacement_dicts


# 银行卡管理
class UserBankAPIView(APIView):
    # 卡片列表
    @request_params_wrapper
    def get(self, *args, request_params=None, detail_id=None, **kwargs):
        # ================== 用户id列表反查询报名 start===============================
        account_params = format_params_handle(
            param_dict=request_params,
            filter_filed_list=["username", 'real_name', 'nickname'],
            is_remove_empty=True
        )

        if account_params:
            account_list, err = UserService.user_list(
                params=account_params)
            if not err:
                # TODO 这里代码很简单的，为什么要调用一个第三方库extract_values啊？ 20241206 by Sieyoo
                # request_params["user_id_list"] = extract_values(account_list['list'], 'user_id')
                request_params["user_id_list"] = [it['user_id'] for it in account_list['list']]

            if isinstance(request_params.get("user_id_list"), list) and len(
                    request_params["user_id_list"]) == 0:
                request_params["user_id_list"] = [0]
        # ================== 用户id列表反查询报名 end ===============================

        data, err = UserBankCardsService.get_bank_card(params=request_params, detail_id=detail_id)
        print(456)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 添加卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def post(self, *args, request_params=None, user_info, **kwargs):
        request_params.setdefault("user_id", user_info.get("user_id"))
        data, err = UserBankCardsService.add(params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response()

    # # 修改卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def put(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or request_params.pop("pk", None)
        data, err = UserBankCardsService.edit(pk=pk, update_params=request_params)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)

    # 删除卡片
    @user_authentication_force_wrapper
    @request_params_wrapper
    def delete(self, *args, request_params=None, **kwargs):
        pk = request_params.pop("id", None) or kwargs.pop("pk", None)
        data, err = UserBankCardsService.delete(pk=pk)
        if err:
            return util_response(err=1000, msg=err)
        return util_response(data=data)



from django.test import TestCase
from ..services.user_service import UserService
from ..services.user_detail_info_service import DetailInfoService
from ..utils.j_ulid_field import JULIDField
import datetime
from ulid import ULID


class TestExtension(TestCase):
    def setUp(self):
        pass

    __right = {
        "account": 'u001',
    }

    # def test_add(self):
    #     result, err = UserService.user_add(self.__right)
    #     print("> UserBaseTest: test_add:", result, err)
    #     self.assertEqual(err, None)

    def test_detail(self):
        exist, err = UserService.user_add(self.__right)
        print("> TestExtension: test_detail 1:", exist, err)
        result, err = DetailInfoService.get_detail(exist['id'])
        print("> TestExtension: test_detail:", result, err)
        self.assertEqual(err, None)

    # def test_edit(self):
    #     UserService.user_add(self.__right)
    #     exist, err = UserService.check_account(self.__right['account'])
    #     print("> UserBaseTest: test_edit: 1 exist:", exist)
    #     exist['fullname'] = '路乙'
    #     result, err = UserService.user_edit(exist, user_id=exist['user_id'])
    #     print("> UserBaseTest: test_edit: 2 result:", result)
    #     self.assertEqual(err, None)

    # def test_delete(self):
    #     UserService.user_add(self.__right)
    #     exist, err = UserService.check_account(self.__right['account'])
    #     print("> UserBaseTest: test_delete: 1 exist:", exist)
    #
    #     result, err = UserService.user_delete(exist['user_id'])
    #     print("> ThreadClassifyTest::test_delete:", result)
    #     self.assertEqual(err, None)


# -*- encoding:utf-8 -*-

class JType:
    @staticmethod
    def is_number(s):
        """判断是否是数字，包括浮点型"""
        try:
            float(s)
            return True
        except ValueError:
            pass

        try:
            import unicodedata
            unicodedata.numeric(s)
            return True
        except (TypeError, ValueError):
            pass

        return False

    @staticmethod
    def is_equal(a, b):
        """判断两个数值是否相等，只支持数字和字符串"""
        print("> is_equal:", a, b)
        aa = str(a).strip() if JType.is_number(a) else a.strip()
        bb = str(b).strip() if JType.is_number(b) else b.strip()
        return aa == bb











# custom_fields.py
from django.db import models
from ulid import ULID
from datetime import datetime

class JULIDField(models.CharField):
    description = "ULID (Universally Unique Lexicographically Sortable Identifier 通用唯一字典排序标识符)"

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 32  # ULID 是 26 个字符的字符串。为了兼容uuid32
        super().__init__(*args, **kwargs)

    @staticmethod
    def get_u12(dt: datetime = None) -> str:
        """
        获取12位ULID。截取ulid26位的第2-10位为时间，第11-13位为随机数
        @param dt: 指定生成时间，精确到毫秒。默认为当前时间
        """
        if not dt:
            dt = datetime.now()
        u12 = str(ULID.from_datetime(dt))[1:13]
        # print('get_u12', u12, dt)
        JULIDField.from_u12(u12)
        return u12.lower()

    @staticmethod
    def from_u12(u12: str) -> ULID:
        if len(u12) != 12:
            return
        u26 = ULID.from_str('0' + u12 + '0000000000000')
        # print('from_u12', u26, u26.datetime)
        return u26


    # @staticmethod
    # def get_u16(dt: datetime = None) -> str:
    #     """
    #     获取12位ULID
    #     @param dt: 指定生成时间，精确到毫秒。默认为当前时间
    #     """
    #     if not dt:
    #         dt = datetime.now()
    #     u16 = str(ULID.from_datetime(dt))[0:16]
    #     return u16
    #
    # @staticmethod
    # def from_u16(u16: str) -> ULID:
    #     if len(u16) != 16:
    #         return
    #     u26 = ULID.from_str(u16 + "0000000000")
    #     # print('from_u12', u26, u26.timestamp)
    #     return u26




# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""

import json


# json 结果集返回
def parse_json(result):
    if not result is None:
        if type(result) is str:
            try:
                result = json.loads(result.replace("'", '"').replace('\\r', "").replace('\\n', "").replace('\\t', "").replace('\\t', ""))
            except Exception as e:
                return result
        if type(result) is list:
            for index, value in enumerate(result):
                result[index] = parse_json(value)
        if type(result) is dict:
            for k, v in result.items():
                result[k] = parse_json(v)
    return result




# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import json
from urllib.parse import parse_qs


# 请求参数解析
def parse_data(request):
    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    content_type = request.META.get('CONTENT_TYPE', "").split(";")[0]
    method = request.method
    if content_type == "text/plain" or method == "GET":
        try:
            body = request.body.decode("utf-8")
            data = json.loads(body)
        except Exception:
            data = request.GET
            if not data:
                data = request.POST
            if not data:
                data = {}
    elif content_type == "application/json":
        return json.loads(request.body)
    elif content_type == "multipart/form-data":
        data = request.POST
    # elif content_type == "application/xml":
    #     try:
    #         data = xmltodict.parse(request.body)
    #         return data.get("body") or data.get("data", {})
    #     except Exception as e:
    #         data = {}
    elif content_type == "application/x-www-form-urlencoded":
        data = parse_qs(request.body.decode())
        if data:
            data = {k: v[0] for k, v in data.items()}
        else:
            data = {}

    else:
        data = getattr(request, 'data', {})
    return {k: v for k, v in data.items()}



# _*_coding:utf-8_*_

import logging

from rest_framework import generics
from rest_framework import serializers
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from ..models import BaseInfo
from ..models import ContactBook
# from apps.finance.models import *
from ..services.contact_book import ContactBookService
from ..services.user_service import UserService

logger = logging.getLogger(__name__)


class UserContactBookSerializer(serializers.ModelSerializer):
    # class Meta:
    #     model = BaseInfo
    #     fields = [
    #         'id',
    #         'fullname',
    #     ]
    class Meta:
        model = ContactBook
        fields = [
            'friend_id',
        ]


# 获取通讯录列表
class UserContactBook(generics.UpdateAPIView):  # 或继承(APIView)
    """ REST framework的APIView实现获取card列表 """
    # authentication_classes = (TokenAuthentication,)  # token认证
    # permission_classes = (IsAuthenticated,)   # IsAuthenticated 仅通过认证的用户
    permission_classes = (AllowAny,)  # 允许所有用户 (IsAuthenticated,IsStaffOrBureau)
    serializer_class = UserContactBookSerializer

    def get(self, request, *args, **kwargs):
        token = self.request.META.get('HTTP_AUTHORIZATION', '')
        self.params = request.data
        if not token:
            return Response({'err': 4001, 'msg': '缺少Token', })

        user_serv, error_text = UserService.check_token(token)
        if error_text:
            return Response({'err': 4001, 'msg': error_text, })
        user_id = user_serv.get('user_id', '')
        if not user_id:
            return Response({'err': 4002, 'msg': 'token验证失败', })

        # user_base_info_list = BaseInfo.objects.all()
        # serializer = UserContactBookSerializer(user_base_info_list, many=True)
        find_user_id = self.params.get('user_id', '')
        if not find_user_id:
            find_user_id = user_id
        list = ContactBookService.get_list(self, find_user_id)
        return list

    def post(self, request, *args, **kwargs):
        token = self.request.META.get('HTTP_AUTHORIZATION', '')
        self.params = request.data
        if not token:
            return Response({'err': 4001, 'msg': '缺少Token', })

        user_id = UserService.check_token(token)
        if not user_id:
            return Response({'err': 4002, 'msg': 'token验证失败', })

        user_id = self.params.get('user_id', '')
        friend_id = self.params.get('friend_id', '')
        remarks = self.params.get('remarks', '')
        data = {
            'user_id': user_id,
            'friend_id': friend_id,
            'remarks': remarks
        }

        add = ContactBookService.add_friends(self, data)
        return add
        # list = ContactBookService.get_list(self, user_id)
        # return list



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections import UserList
from typing import TYPE_CHECKING

from .jwk import AbstractJWKBase, jwk_from_dict

if TYPE_CHECKING:
    UserListBase = UserList[AbstractJWKBase]
else:
    UserListBase = UserList


class JWKSet(UserListBase):

    def filter_keys(self, kid=None, kty=None):
        # When "kid" values are used within a JWK Set, different
        # keys within the JWK Set SHOULD use distinct "kid" values.  (One
        # example in which different keys might use the same "kid" value is if
        # they have different "kty" (key type) values but are considered to be
        # equivalent alternatives by the application using them.)

        if kid and kty:
            return [key for key in self.data
                    if key.get_kty() == kty and key.get_kid() == kid]
        if kid:
            return [key for key in self.data if key.get_kid() == kid]
        if kty:
            return [key for key in self.data if key.get_kty() == kty]

        return self.data.copy()

    def to_dict(self, public_only=True):
        keys = [key.to_dict(public_only=public_only) for key in self.data]
        return {'keys': keys}

    @classmethod
    def from_dict(cls, dct):
        keys = [jwk_from_dict(key_dct) for key_dct in dct.get('keys', [])]
        return cls(keys)



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import datetime
from .j_transform_type import JTransformType

# 处理字典：白名单、黑名单、别名、移除空值、筛选类型
def format_params_handle(
        param_dict: dict,
        filter_filed_list: list = None,
        remove_filed_list: list = None,
        alias_dict: dict = None,
        split_list: list = None,
        split_char: str = ";",
        is_remove_null: bool = True,
        is_remove_empty: bool = False,
        date_format_dict: dict = None,
        is_validate_type: bool = False,
) -> dict:
    """
    字段筛选并替换成别名
    :param is_validate_type: 是否验证数据合法性
    :param param_dict: 参数值
    :param filter_filed_list: 字段白名单，如:["id":int]
    :param remove_filed_list: 字段黑名单,["id",....]
    :param alias_dict: 别名字典, {"id":"user_id"}
    :param split_char: 字符串拆分依据
    :param split_list: 拆分字符换,该参数为了适配使用符号分割成列表。["id_list",....]
    :param is_remove_null:  是否把带有None的值移除掉
    :param is_remove_empty: 是否移除value类型等于False的字段,移除"",0,"0",None
    :param date_format_dict: 日期格式化 日期列表，{field：(from_format,to_format),}如：{"field":("%Y-%m-%d %H:%M:%S","%Y-%m-%d %H:%M:%S")}
    :return: param_dict
    """
    # 转换的数据类型不符合，直接返回出去
    if not isinstance(param_dict, dict):
        raise ValueError("param_dict 必须是字典格式")
    # 过滤字段并且得数据类型映射
    new_filter_filed_list = []  # 拆分字段类型的白名单
    must_type_map = {}  # 从白名单中拆分出字段与类型的映射
    if filter_filed_list and isinstance(filter_filed_list, list):
        for i in filter_filed_list:
            key_type_list = i.split("|", 1)
            [key, key_type] = key_type_list if len(key_type_list) == 2 else [key_type_list[0], None]  # 如果为类型None，则默认是字符串
            must_type_map[key] = key_type
            new_filter_filed_list.append(key)
        param_dict = {k: v for k, v in param_dict.copy().items() if k in new_filter_filed_list and (not is_remove_empty or v) and (not is_remove_null or not v is None)}
    # 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_dict = {k: v for k, v in param_dict.copy().items() if not k in remove_filed_list and (not is_remove_empty or v) and (not is_remove_null or not v is None)}
    # 字段拆分
    if split_list and isinstance(split_list, list):
        param_dict = {k: (str(v).split(split_char) if k in split_list and not isinstance(v, list) else v) for k, v in param_dict.copy().items()}
    # 类型转换
    if must_type_map and isinstance(must_type_map, dict):
        for k, v in param_dict.copy().items():
            v, is_err = JTransformType.to(v, must_type_map.get(k, None))
            if is_err and not is_validate_type:
                param_dict.pop(k)
            elif is_err and is_validate_type:
                raise ValueError(k + ":" + is_err)
            else:
                # 覆盖掉转换的值
                param_dict[k] = v

    # 日期字段格式转换
    if date_format_dict and isinstance(date_format_dict, dict):
        for k, v in param_dict.copy().items():
            if not date_format_dict.get(k):
                continue
            try:
                from_format, to_format = date_format_dict[k]
                if not isinstance(v, datetime.datetime) and isinstance(v, str):
                    parse_date_obj = datetime.datetime.strptime(v, from_format)
                else:
                    parse_date_obj = v
                param_dict[k] = parse_date_obj.strftime(to_format)
            except Exception as e:
                pass

    # 别名替换
    if alias_dict and isinstance(alias_dict, dict):
        param_dict = {alias_dict.get(k, k): v for k, v in param_dict.copy().items()}

    return param_dict



import sys
from django.forms import model_to_dict

if 'xj_role' in sys.modules:
    from xj_role.services.user_group_service import UserGroupService
from ..models import Platform, UserSsoServe
from ..services.login_service import LoginService
from ..services.user_service import UserService
from ..utils.write_to_log import write_to_log
from ..utils.utility_method import generate_password, replace_placeholders
from xj_user.services.user_relate_service import UserRelateToUserService


class UserMainService:

    # @staticmethod
    # def register(params):
    #     platform_id = params.get("platform_id", None)  # 平台。不应该支持ID传入，无法数据移植。20230507 by Sieyoo
    #     platform_code = params.get("platform_code", None)
    #     account = params.get('account', '')
    #     nickname = params.get('nickname', '')
    #     phone = params.get('phone', '')
    #     password = str(params.get('password', ''))
    #
    #     if platform_code:
    #         platform_set = Platform.objects.filter(platform_code=platform_code).first()
    #         if not platform_set:
    #             return None, "platform不存在平台名称：" + platform_code
    #         platform_id = model_to_dict(platform_set)['platform_id']
    #
    #     if platform_id:
    #         platform_set = Platform.objects.filter(platform_id=platform_id).first()
    #         if not platform_set:
    #             return None, "所属平台不存在"
    #         platform_code = model_to_dict(platform_set)['platform_code']
    #
    #     account_serv, error_text = UserService.check_account(account, platform_code, is_registered=False)
    #     if error_text:
    #         return None, error_text
    #
    #     register_serv, register_error = LoginService.register_write(account, nickname, phone, platform_id,
    #                                                                 platform_code, password)
    #     if register_error:
    #         return None, register_error
    #
    #     return register_serv, None

    """
     登录整合接口，支持以下几种登录方式：
          1、PASSWORD 账户密码登录
          2、SMS 短信验证码登录 （支持多账号登录*）
          3、WECHAT_APPLET 微信小程序授权登录
          4、WECHAT_WEB 微信公众号授权登录
          5、WECHAT_APP 微信APP授权登录

    """

    @staticmethod
    def login_integration_interface(params):
        data = {}
        # ----------------------------获取信息----------------------------------------
        # TODO platform_id字段 即将弃用，改为platform_code 20230507 by Sieyoo
        platform_id = params.get("platform_id", None)  # 平台。不应该支持ID传入，无法数据移植。20230507 by Sieyoo
        platform_code = params.get("platform_code", None)
        user_id = params.get("user_id", None)  # 用户id
        login_type = params.get("login_type", None)  # 支持的登录方式
        openid_code = params.get("openid_code", None) or params.get("code", None)  # 微信登录code
        phone_code = params.get("phone_code", None)  # 微信手机号code
        sms_code = params.get("sms_code", None)  # 短信验证码
        # sso_serve_id = params.get("sso_serve_id", None)  # 单点登录用户平台，弃用字段设计
        sso_serve_code = params.get("sso_serve_code", None)  # 单点登录用户平台
        phone = params.get("phone", None)  # 手机号
        other_params = params.get("other_params", None)
        account = params.get("account", None)  # 账户
        password = params.get("password", None)  # 密码
        bind_data = params.get("bind_data", None)  # 绑定的数据
        apple_logo = params.get("apple_logo", None)  # 苹果
        # ------------------------边界检查----------------------------------------------
        if not login_type:
            return None, "登录方式不能为空"
        if platform_code:
            platform_set = Platform.objects.filter(platform_code=platform_code).first()
            if not platform_set:
                return None, "platform不存在平台名称：" + platform_code
            platform_id = model_to_dict(platform_set)['platform_id']

        if platform_id:
            platform_set = Platform.objects.filter(platform_id=platform_id).first()
            if not platform_set:
                return None, "所属平台不存在"
            platform_code = model_to_dict(platform_set)['platform_code']

        # 平台ID不具有可识别性，禁止传此参数 by Sieyoo at 20231219
        # if sso_serve_id:
        #     sso_server_set = UserSsoServe.objects.filter(id=sso_serve_id).first()
        #     if not sso_server_set:
        #         return None, "单点登录平台不存在：" + str(sso_serve_id)
        #     sso_serve_id = model_to_dict(sso_server_set)['id']

        sso_serve_id = None
        if sso_serve_code:
            sso_server_set = UserSsoServe.objects.filter(sso_code=sso_serve_code).first()
            if not sso_server_set:
                return None, "单点登录平台不存在：" + sso_serve_code
            sso_serve_id = model_to_dict(sso_server_set)['id']
        # ------------------------登录类型判断----------------------------------------------

        if other_params is None:
            other_params = {}

        # try:

        current_user, user_err = LoginService.type_judgment(
            login_type=login_type, account=account, phone=phone, password=password, platform_code=platform_code,
            sms_code=sms_code, user_id=user_id, openid_code=openid_code, phone_code=phone_code, bind_data=bind_data,
            apple_logo=apple_logo)
        # print("> login_integration_interface: current_user, user_err:", current_user, user_err)
        if user_err:
            return None, user_err

            # 设计错误，严禁在函数之间传递实例（尤其是模型实例），转为字典对象 by Sieyoo at 20231219
            # if isinstance(current_user.get("user_info", None), list):
            #     return current_user, None
            # else:

            # 登录逻辑处理
            # data, err = LoginService.logical_processing(current_user.get("user_info", None),
            #                                             current_user.get("phone", None),
            #                                             current_user.get("appid", None),
            #                                             current_user.get("openid", None),
            #                                             current_user.get("unionid", None),
            #                                             platform_id,
            #                                             platform_code, other_params,
            #                                             current_user.get("wx_login_type", None))
            # if err:
            #     return None, err

        # except Exception as e:
        #     write_to_log(
        #         prefix="登录异常",
        #         content='---用户登录异常：' + str(e) + '---',
        #         err_obj=e
        #     )

        # ------------------------登录逻辑处理的移植代码（LoginService.logical_processing）------------------------

        # 如果不存在则注册
        user_info = current_user.get("user_info", None)
        is_create = False
        if not user_info:
            user_info = LoginService.register_phone_account(phone=current_user.get("phone", None),
                                                            platform_code=platform_code)
            is_create = True

        # 检查单点登录信息（查询并创建）针对小程序、公众号、APP，同一微信平台绑定过的绑定
        if sso_serve_id:
            sso_set, sso_err = LoginService.sso_verify(
                sso_serve_id=sso_serve_id, user_id=user_info.get('id', ""), appid=current_user.get("appid", None),
                is_exist=True, sso_unicode=current_user.get("openid", None),
                union_code=current_user.get("unionid", None))
            if sso_err:
                return None, sso_err

        # TODO token可以考虑让各个子服务独立获取token，而不是公共生成Token，当然，这样设计好不好有待商考 20230507 by Sieyoo
        token = LoginService.make_token(user_info.get('uuid', ""), user_info.get("username", ""), platform_id,
                                        platform_code)
        password = generate_password(12)
        # 修改用户登录信息，绑定token
        auth_set, auth_err = LoginService.bind_token(user_id=user_info.get('id', ""), token=token,
                                                     is_create=is_create, password=password)
        if auth_err:
            return None, auth_err

        # 登录逻辑处理-------本代码方法注掉，平铺到下方
        # data, err = LoginService.logical_processing(current_user.get("user_info", None),
        #                                             current_user.get("phone", None),
        #                                             current_user.get("appid", None),
        #                                             current_user.get("openid", None),
        #                                             current_user.get("unionid", None),
        #                                             platform_id,
        #                                             platform_code, other_params,
        #                                             current_user.get("wx_login_type", None))
        # if err:
        #     return None, err

        wx_login_type = ""  # 登录类型
        # 代码理解：如果是新用户，注册成功并发送短信通知
        if is_create:
            # 注册成功后发送短信通知
            sms_data = {
                "phone": phone,
                "platform": 'ALi',
                "account": user_info.get("username", ""),
                "pwd": password,
                "type": "PWD"
            }

            if wx_login_type == "applet" or wx_login_type == "sms":
                sms_set, sms_err = SmsService.bid_send_sms(sms_data)
                if sms_err:
                    write_to_log(
                        prefix="首次登录写入用户详细信息异常",
                        content='---首次登录写入用户详细信息异常：' + str(sms_err) + '---',
                        err_obj=sms_err
                    )

            try:
                other_params.setdefault("user_id", user_info.get('id', ""))
                other_params.setdefault("score", "5")  # 用户评分初始化，镖行天下业务逻辑 TODO 后期业务抽离，路程控制
                data, detail_err = DetailInfoService.create_or_update_detail(other_params)
                if detail_err:
                    raise Exception(detail_err)
            except Exception as e:
                write_to_log(
                    prefix="首次登录写入用户详细信息异常",
                    content='---首次登录写入用户详细信息异常：' + str(e) + '---',
                    err_obj=e
                )
            # 用户第一次登录即注册，绑定用户的分组ID
            try:
                group_id = other_params.get("group_id")
                if group_id:
                    data, err = UserGroupService.user_bind_group(user_id=user_info.get('id', ""), group_id=group_id)
                    write_to_log(
                        prefix="group_id:" + str(other_params.get("group_id", "")) + "绑定部门ID异常",
                        content=err
                    )
            except Exception as err:
                write_to_log(
                    prefix="绑定部门ID异常",
                    content="group_id:" + str(other_params.get("group_id", "")),
                    err_obj=err
                )
        # 代码理解：否则是已注册过的用户，同时则将其邀请关系进行绑定？
        else:
            # 绑定用户关系 邀请关系和收益关系
            data, relate_err = UserRelateToUserService.bind_bxtx_relate(params=other_params, user_info=user_info)
            if relate_err:
                write_to_log(
                    prefix='绑定用户关系异常：' + str(relate_err),
                    content='当前用户ID:' + str(user_info.get("id", "")) + '\n detail_params:' + json.dumps(
                        other_params),
                    err_obj=relate_err
                )

        return {'token': "Bearer " + auth_set.token, 'user_info': user_info}, None



# _*_coding:utf-8_*_

import sys
import re
from datetime import datetime, timedelta
from pathlib import Path
from typing import Tuple, Dict, Any
# import uuid as uuid
import jwt

from django.conf import settings
from django.contrib.auth.hashers import check_password
from django.contrib.auth.hashers import make_password
from django.core.paginator import Paginator, EmptyPage
from django.db import transaction
from django.db.models import Q, F
from django.forms import model_to_dict

from rest_framework import serializers

from ..models import Auth, Platform, ExtendField, DetailInfo
from ..models import BaseInfo
# from ..utils.j_config import JConfig
from ..utils.j_dict import JDict
from ..utils.custom_tool import filter_result_field, format_params_handle, format_list_handle
from ..utils.get_short_id import get_short_id
from ..utils.j_join_list import JJoinList
from ..utils.j_ulid_field import JULIDField
from ..utils.nickname_generate import gen_one_word_digit


# 获取配置
module_config = JDict(settings.CONFIG.get('xj_user'))

app_id = module_config.app_id or ""
app_secret = module_config.secret or ""
jwt_secret_key = module_config.jwt_secret_key or "@zxmxy2021!"
expire_day = module_config.expire_day or ""
expire_second = module_config.expire_second or ""


class UserInfoSerializer(serializers.ModelSerializer):
    # 方法一：使用SerializerMethodField，并写出get_platform, 让其返回你要显示的对象就行了
    # p.s.SerializerMethodField在model字段显示中很有用。
    # platform = serializers.SerializerMethodField()

    # # 方法二：增加一个序列化的字段platform_name用来专门显示品牌的name。当前前端的表格columns里对应的’platform’列要改成’platform_name’
    user_id = serializers.ReadOnlyField(source='id')

    # permission_value = serializers.SerializerMethodField()

    # platform_id = serializers.ReadOnlyField(source='platform.platform_id')
    # platform_name = serializers.ReadOnlyField(source='platform.platform_name')

    class Meta:
        model = BaseInfo
        fields = [
            'user_id',
            # 'platform',
            # 'platform_uid',
            # 'platform__platform_name',
            # 'platform_id',
            # 'platform_name',
            # 'get_group_desc',
            'username',
            'fullname',
            'phone',
            'email',
            # 'wechat_openid',
            'user_info',
            # 'user_group',
            # 'user_group_id',
            # 'permission',
            # 'permission_value'
        ]
        # exclude = ['platform_uid']

    # def get_permission_value(self, instance):
    #     pass

    # # 这里是调用了platform这个字段拼成了get_platform
    # def get_platform(self, obj):
    #     return obj.platform.platform_name
    #     # return {
    #     #     'id': obj.platform.platform_id,
    #     #     'name': obj.platform.platform_name,
    #     # }


class UserService:
    # 临时
    detail_fields = ['user_id','real_name', 'sex', 'birth', 'tags', 'signature', 'cover', 'id_card_type', 'id_card_no',
                     'language', 'region_code', 'more']

    # 用户基础字段。方法二：[i.name for i in BaseInfo._meta.fields]，该方法对特定字段不能显式处理，如id, uuid, is_delete。
    base_fields = [
        'user_id', 'user_uuid', 'user_no', 'username', 'nickname', 'fullname',
        'phone', 'email', 'avatar', 'user_type', 'user_vip', 'user_level', 'user_info',
        'privacies', 'is_auth', 'is_using'
    ]

    def __init__(self):
        pass

    # 检测账户
    @classmethod
    def detail(cls, uuid: str):
        """
        @param uuid 用户UUID。
        """
        # print("> UserService::detail:", uuid)
        user_set = BaseInfo.objects
        user_set = user_set.annotate(user_id=F("id"), user_uuid=F("uuid")).extra(
            select={
                "user_uuid": 'REPLACE(uuid, "-", "")',
            }
        )
        # print("> check_account user_set query:", user_set.query)
        # print("> UserService::detail user_set:", uuid, user_set)
        user_dict = user_set.values(*cls.base_fields).first()
        # print("> UserService::detail user_dict:", uuid, user_dict)
        if not user_dict:
            return None, "账户不存在"
        return user_dict, None

    @classmethod
    def check_account(cls, account: str, platform_code: str = None, is_registered: bool = True) -> Tuple[dict, str]:
        """
        检测并返回账户信息。（自动判断账户类型；自动判断多账号冲突）

        @param account: 用户账户，支持三种类型：手机、用户名、邮箱。自动判断
        @param platform_code: 用户平台码，4位大写字母
        @param is_registered: 是否判断账户已注册。如果为真，账号不存在则报错；为假，账号不存在返回Null
        @description: 注意：用户名不推荐由纯数字构成，因为那样容易和11位手机号冲突
        """
        # 账号类型判断
        if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
            account_type = 'phone'
        elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
            account_type = 'email'
        elif re.match(r'^[A-Za-z\u4E00-\u9FA5_0-9]\w*$', account):
            account_type = 'username'
        else:
            return None, f"账号({account})必须是用户名、手机或者邮箱，其中用户名不能少于4个字符且不能以数字开头，不含特殊字符"

        # 排除注销用户
        user_query = BaseInfo.objects.exclude(is_delete=True)

        # 根据类型查询
        if account_type == 'phone':
            user_query = user_query.filter(phone=account)
        elif account_type == 'email':
            user_query = user_query.filter(email=account)
        elif account_type == 'username':
            user_query = user_query.filter(username=account)
        else:
            return None, "账号类型匹配失败。原则上这是一个不可能出现的提示！"

        # 删除UUID横杠
        user_query = user_query.annotate(user_id=F("id")).extra(
            select={"user_uuid": 'REPLACE(uuid, "-", "")', }
        )

        # 是否检查平台码
        if platform_code:
            user_query = user_query.filter(platforms_to_users__platform_code=platform_code)

        # 请求数据库
        user_list = user_query.values(*cls.base_fields)

        # 多账号冲突
        if len(user_list) > 1:
            return None, "登录异常，请联系管理员，发现多账号冲突"

        # 检查账号存在
        user_set = user_list.first()
        if not user_set:
            return None, "账户不存在" if is_registered else None

        return user_set, None

    # 检测令牌
    @classmethod
    def check_token(cls, token):
        """
        @param token 用户令牌。
        @description 注意：用户令牌的载荷体payload中必须包含两个参数：account账号、exp过期时间，其中账号可以是手机、用户名、邮箱三种。
        @description BEARER类型的token是在RFC6750中定义的一种token类型，OAuth2.0协议RFC6749对其也有所提及，算是对RFC6749的一个补充。BEARER类型token是建立在HTTP/1.1版本之上的token类型，需要TLS（Transport Layer Security）提供安全支持，该协议主要规定了BEARER类型token的客户端请求和服务端验证的具体细节。
        @description 理论上，每次请求令牌后就更新一次令牌，以监测用户长期访问时，不至于到时间后掉线反复登陆。
        """
        # 检查是否有Bearer前辍，如有则截取
        # print("> token 1:", token)
        if not token:
            return None, "请登录"  # 缺少Token

        if re.match(r'Bearer(.*)$', token, re.IGNORECASE):
            token = re.match(r'Bearer(.*)$', token, re.IGNORECASE).group(1).strip()
        # print("> token 2:", token)

        if not token:
            return None, "您尚未登录"

        # # 验证token。另一种方式，从数据库核对Token，通过对比服务端的Token，以确定是否为服务器发送的。今后启用该功能。
        # auth_set = Auth.objects.filter(Q(token=token)).order_by('-update_time')
        # # print("> auth:", auth_set)
        # if not auth_set.count():
        #     raise MyApiError('token验证失败', 6002)
        # auth = auth_set.first()

        try:
            # jwt.decode会自动检查exp参数，如果超时则抛出jwt.ExpiredSignatureError超时
            # jwt_payload = jwt.decode(token, key=Config.getIns().get('xj_user', 'JWT_SECRET_KEY'), verify=True, algorithms=["RS256", "HS256"])
            jwt_payload = jwt.decode(token, key=jwt_secret_key, verify=True, algorithms=["RS256", "HS256"])
            # print("> jwt_payload:", jwt_payload)

        except jwt.ExpiredSignatureError:
            return None, "登录已过期，请重新登录"

        except jwt.InvalidTokenError:
            return None, "用户令牌无效，请重新登录"

        account = jwt_payload.get('account', None)
        user_uuid = jwt_payload.get('user_uuid', None)
        user_id = jwt_payload.get('user_id', None)
        platform_id = jwt_payload.get('platform_id', None)
        if not account:
            return {'payload': jwt_payload}, "错误：令牌载荷中未提供用户账户Account"

        # 检测用户令牌时不应该调用用户信息，这会导致任何接口都会查询用户表，时间会增加
        # user_info = UserService.check_account(account=account)

        return {'account': account, 'user_uuid': user_uuid, 'user_id': user_id, 'platform_id': platform_id}, None

    # @staticmethod
    # def user_add(params):
    #     """
    #     用户添加服务
    #     :param params: 添加参数
    #     :return: data,err
    #     """
    #     try:
    #         account = str(params.get('account', ''))
    #         password = str(params.get('password', ''))
    #         fullname = str(params.get('fullname', ''))
    #         nickname = str(params.get('nickname', ''))
    #         # 边界检查
    #         if not account:
    #             return None, "account必填"
    #         # 账号类型判断
    #         if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
    #             account_type = 'phone'
    #         elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
    #             account_type = 'email'
    #         elif re.match(r'^[A-z\u4E00-\u9FA5]+\w*$', account):
    #             account_type = 'username'
    #         else:
    #             return None, "账号必须是手机、邮箱格式或者为纯英文且不含特殊字符12位字符"
    #         # 检查账号是否存在
    #         user_list = None
    #         if account_type == 'phone':
    #             user_list = BaseInfo.objects.filter(Q(phone=account))
    #         elif account_type == 'email':
    #             user_list = BaseInfo.objects.filter(Q(email=account))
    #         elif account_type == 'username':
    #             user_list = BaseInfo.objects.filter(Q(username=account))
    #
    #         if user_list.count() and account_type == 'phone':
    #             return None, "手机已被注册: " + account
    #         elif user_list.count() and account_type == 'email':
    #             return None, "邮箱已被注册: " + account
    #         elif user_list.count() and account_type == 'username':
    #             return None, "用户名已被注册: " + account
    #
    #         # 创建基础信息
    #         base_info = {
    #             'username': account if account_type == 'username' else get_short_id(),
    #             'phone': account if account_type == 'phone' else "",
    #             'email': account if account_type == 'email' else '',
    #             'fullname': fullname,
    #             "nickname": nickname,
    #         }
    #         current_user = BaseInfo.objects.create(**base_info)
    #         # 创建登录信息
    #         token = jwt.encode({'account': account}, jwt_secret_key)
    #         auth = {
    #             'user_id': current_user.id,
    #             'password': make_password(password, None, 'pbkdf2_sha1'),
    #             'plaintext': password,
    #             'token': token,
    #         }
    #         Auth.objects.create(**auth)
    #         return {"user_id": current_user.id}, None
    #     except SyntaxError:
    #         return None, "语法错误"
    #     except LookupError:
    #         return None, "无效数据查询"
    #     except Exception as valueError:
    #         return None, valueError.msg if hasattr(valueError, 'msg') else valueError.args
    #     except:
    #         return None, "未知错误"
    @classmethod
    def user_add(cls, params):
        """
        用户添加服务
        :param params: 添加参数
        :return: data,err
        """
        try:
            account = str(params.get('account', get_short_id()))
            password = str(params.get('password', '123456'))
            fullname = str(params.get('fullname', ''))
            nickname = str(params.get('nickname', gen_one_word_digit()))
            user_type = str(params.get('user_type', "PERSON"))
            phone = str(params.get('phone', ""))
            email = str(params.get('email', ""))

            # 账号类型判断
            if re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', account):
                account_type = 'phone'
            elif re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', account):
                account_type = 'email'
            elif re.match(r'^[A-Za-z\u4E00-\u9FA5_0-9]\w*$', account):
                account_type = 'username'
            else:
                return None, "账号必须是手机、邮箱格式或者为纯英文且不含特殊字符12位字符"

            # print("账号类型", account_type)
            # 检查账号是否存在
            user_list = None
            if account_type == 'phone':
                user_list = BaseInfo.objects.filter(Q(phone=account))
            elif account_type == 'email':
                user_list = BaseInfo.objects.filter(Q(email=account))
            elif account_type == 'username':
                user_list = BaseInfo.objects.filter(Q(username=account))

            if user_list.count() and account_type == 'phone':
                return None, "手机已被注册: " + account
            elif user_list.count() and account_type == 'email':
                return None, "邮箱已被注册: " + account
            elif user_list.count() and account_type == 'username':
                return None, "用户名已被注册: " + account

            # 校验手机号格式
            if phone and not re.match(r'(^1[356789]\d{9}$)|(^\+?[78]\d{10}$)', phone):
                return None, "手机号格式不正确"

            # 校验邮箱格式
            if email and not re.match(r'^\w+[\w\.\-\_]*@\w+[\.\w]*\.\w{2,}$', email):
                return None, "邮箱格式不正确"

            # 创建基础信息
            base_info = {
                'username': account if account_type == 'username' else get_short_id(),
                'phone': phone if phone else None,
                'email': email if email else None,
                'fullname': fullname,
                "nickname": nickname,
                "uuid": JULIDField.get_u12(),
                "user_type": user_type
            }
            current_user = BaseInfo.objects.create(**base_info)

            # 创建登录信息
            token = jwt.encode({'account': account}, jwt_secret_key)
            auth = {
                'user_id': current_user.id,
                'password': make_password(password, None, 'pbkdf2_sha1'),
                'plaintext': password,
                'token': token,
            }
            Auth.objects.create(**auth)
            print(f"current_user.to_json():{current_user.to_json()}")
            return current_user.to_json(), None
        except SyntaxError:
            return None, "语法错误"
        except LookupError:
            return None, "无效数据查询"
        except Exception as valueError:
            return None, valueError.msg if hasattr(valueError, 'msg') else valueError.args
        except:
            return None, "未知错误"

    @classmethod
    def bind_user_level(cls, user_id, user_level: int = None):
        print("data", user_id, user_level)
        if not user_id:
            return None, "用户ID不存在"
            # 更新基础信息
        base_query_set = BaseInfo.objects.filter(id=user_id)
        if not base_query_set.first():
            return None, "用户不存在"
            # 指定要更新的字段及其新值
        update_fields = {
            'user_level': user_level,
        }
        try:
            base_query_set.update(**update_fields)
        except Exception as e:
            return None, str(e)

        return None, None

    @classmethod
    def user_edit(cls, params=None, user_id=None, user_uuid=None):
        """
        用户基础信息修改服务。
        PS: 不允许修改用户的用户名，用户名负责登录作用。
        :param params: 修改参数集合，包含用户的基础信息和详细信息。
        :param user_id: 用户ID，用于定位要修改的用户。
        :param user_uuid: 用户UUID，用于定位要修改的用户。
        :return: 返回修改后的用户信息或错误信息。
        """
        if params is None:
            params = {}
        if not user_id and not user_uuid:
            return None, "用户ID或UUID不存在"

        # 绑定信息验重
        email_other_user = BaseInfo.objects.exclude(id=user_id).filter(email=params.get("email", None)).first()
        if params.get("email") and email_other_user:
            return None, "当前邮箱被绑定"

        # 用户基础信息中仅仅可以修改以下参数
        base_info_params = format_params_handle(
            param_dict=params.copy(),
            filter_filed_list=["fullname", "nickname", "email", "user_info", "privacies", "phone", "is_using",
                               "avatar"],
            is_remove_null=True
        )

        # 修改详情信息
        field_map_list = ExtendField.objects.all().values("field", 'field_index')
        detail_fields = cls.detail_fields.copy()
        for field_map in field_map_list:
            detail_fields.append(field_map.get("field"))

        # 将拓展字段添加到详细字段
        detail_params = format_params_handle(
            param_dict=params.copy(),
            filter_filed_list=detail_fields,
            is_remove_null=True
        )

        # 拓展字段映射
        field_mapping = {item['field']: item['field_index'] for item in field_map_list}
        detail_params = {field_mapping.get(k, k): v for k, v in detail_params.items()}

        try:
            with transaction.atomic():
                # 根据 user_id 或 user_uuid 查询用户
                if user_id:
                    base_query_set = BaseInfo.objects.filter(id=user_id)
                else:
                    base_query_set = BaseInfo.objects.filter(uuid=user_uuid)

                base_set = base_query_set.first()
                if not base_set:
                    return None, "用户不存在"

                # 修改基础信息
                base_query_set.update(**base_info_params)

                # 判断是否添加过
                detail_user_obj = DetailInfo.objects.filter(user_id=base_set.id)
                if not detail_user_obj.first():
                    # 没有添加，进行添加操作
                    detail_params.pop("id", None)  # 添加的时候不能有ID主键，防止主键冲突
                    detail_params['user_id'] = base_set.id  # Ensure user_id is set
                    detail_set = DetailInfo.objects.create(**detail_params)
                else:
                    # 添加过进行更新
                    detail_user_obj.update(**detail_params)

                # 重新查询更新后的数据
                updated_base_set = BaseInfo.objects.filter(id=base_set.id).first()
                updated_detail_set = DetailInfo.objects.filter(user_id=base_set.id).first()

                # 合并结果
                result = {**model_to_dict(updated_base_set), **model_to_dict(updated_detail_set)}
                return result, None

        except BaseInfo.DoesNotExist as e:
            return None, f"基础信息修改失败:{e}"
        except DetailInfo.DoesNotExist as e:
            return None, f"详细信息修改失败:{e}"
        except Exception as e:
            # 处理其他可能的异常
            return None, f"修改失败:{e}"

    @classmethod
    def user_delete(cls, user_id=None, user_uuid=None):
        """
        删除用户（软删除）。
        支持通过 user_id 或 user_uuid 定位用户。
        :param user_id: 用户ID
        :param user_uuid: 用户UUID
        :return: 返回删除后的用户信息或错误信息。
        """
        if not user_id and not user_uuid:
            return None, "必须提供 user_id 或 user_uuid"

        # 通过 user_id 或 user_uuid 查找用户
        if user_id:
            user_set = BaseInfo.objects.filter(id=user_id).first()
        else:
            user_set = BaseInfo.objects.filter(uuid=user_uuid).first()

        if not user_set:
            return None, f'用户不存在 (user_id: {user_id}, user_uuid: {user_uuid})'

        # 软删除：将 is_delete 设置为 False
        user_set.is_delete = False
        user_set.save()

        # 返回删除后的用户信息
        return user_set.to_json(), None

    @classmethod
    def user_search(cls, user_id_list: list = None, search_params: dict = None, need_map: bool = False,
                    filter_fields: list = None, page: int = 1, page_size: int = 10, order_by: str = None):
        """
        搜索用户，返回用户基础信息列表，支持分页和排序。
        :param user_id_list: 用户id列表
        :param search_params: 搜索字段
        :param need_map: 是否需要映射
        :param filter_fields: 需要查询的字段
        :param page: 当前页码
        :param page_size: 每页大小
        :param order_by: 排序字段，如 'username' 或 '-register_time'
        :return: data, err
        """
        print(not user_id_list and not search_params)
        if not user_id_list and not search_params:
            return [], "参数错误,无法筛选数据"

        # 列表的查询参数处理
        if filter_fields is None:
            filter_fields = ["id", "username", "fullname", "nickname", "phone", "email", "register_time",
                             "user_type", "user_vip", "user_level"]
        else:
            filter_fields = format_list_handle(
                param_list=filter_fields,
                filter_filed_list=["id", "username", "fullname", "nickname", "phone", "email", "register_time",
                                   "user_type", "user_vip", "user_level"]
            )
            filter_fields = ["id", "username", "fullname", "nickname", "phone", "email",
                             "register_time"] if not filter_fields else filter_fields

        # 字段查询参数处理
        if search_params:
            search_params = format_params_handle(
                param_dict=search_params,
                filter_filed_list=["id", "user_id", "username", "fullname", "nickname", "phone", "email",
                                   "register_time", "register_time_start", "register_time_end", "user_type",
                                   "user_vip", "user_level"],
                alias_dict={"user_id": "id", "register_time_start": "register_time__gte",
                            "register_time_end": "register_time_lte"}
            )

        # 查询管理
        try:
            # orm 查询
            if user_id_list:
                user_base_set = BaseInfo.objects.filter(id__in=user_id_list)
            else:
                user_base_set = BaseInfo.objects.filter(**search_params)

            # 排序
            if order_by:
                user_base_set = user_base_set.order_by(order_by)

            # 分页
            paginator = Paginator(user_base_set, page_size)
            try:
                user_base_page = paginator.page(page)
            except EmptyPage:
                return [], "页码超出范围"

            # 获取列表并字段转换，id ==>> user_id
            user_base_list = list(user_base_page.object_list.values(*filter_fields))
            user_base_list = filter_result_field(
                result_list=user_base_list,
                alias_dict={"id": "user_id"}
            )

            # 如果需要转换，则进行映射
            if need_map:
                user_base_list = {i['user_id']: i for i in user_base_list}

            # 返回分页信息
            pagination_info = {
                "total": paginator.count,
                "page": page,
                "page_size": page_size,
                "total_pages": paginator.num_pages,
            }

            return {"data": user_base_list, "pagination": pagination_info}, None
        except Exception as e:
            return [], "err:" + str(e)


from django.test import TestCase
from ..services.user_service import UserService
from ..models import BaseInfo, ExtendField, DetailInfo
from ..utils.j_ulid_field import JULIDField

class TestUserEdit(TestCase):
    def setUp(self):
        # 初始化测试数据
        self.user = BaseInfo.objects.create(
            username="testuser",
            fullname="Test User",
            nickname="testnick",
            email="test@example.com",
            phone="1234567890",
            user_info="{}",
            privacies="{}",
            is_using=True,
            uuid=JULIDField.get_u12()
        )

        # 初始化ExtendField数据
        ExtendField.objects.create(
            field_index="field_1",
            field="school_id",
            type="int",
            description="学校id"
        )
        ExtendField.objects.create(
            field_index="field_2",
            field="grade",
            type="int",
            description="年级"
        )

    def test_user_edit_base_info(self):
        # 测试更新基础信息
        params = {
            "fullname": "Updated User",
            "nickname": "updatednick",
            "email": "updated@example.com",
            "phone": "0987654321",
            "is_using": False
        }
        result, err = UserService.user_edit(params=params, user_id=self.user.id)
        self.assertEqual(err, None)
        self.assertEqual(result["fullname"], "Updated User")
        self.assertEqual(result["nickname"], "updatednick")
        self.assertEqual(result["email"], "updated@example.com")
        self.assertEqual(result["phone"], "0987654321")
        self.assertEqual(result["is_using"], False)

    def test_user_edit_detail_info(self):
        # 测试更新详细信息
        params = {
            "school_id": 123,
            "grade": 10
        }
        result, err = UserService.user_edit(params=params, user_id=self.user.id)
        self.assertEqual(err, None)

        # 验证详细信息
        detail_info = DetailInfo.objects.filter(user_id=self.user.id).first()
        self.assertIsNotNone(detail_info)
        self.assertEqual(detail_info.field_1, '123')
        self.assertEqual(detail_info.field_2, '10')

    def test_user_edit_with_invalid_user_id(self):
        # 测试使用无效的用户ID进行更新
        params = {
            "fullname": "Invalid User"
        }
        result, err = UserService.user_edit(params=params, user_id=9999)  # 假设9999是一个不存在的用户ID
        self.assertIsNotNone(err)
        self.assertEqual(err, "用户不存在")

    def test_user_edit_with_duplicate_email(self):
        # 测试更新时使用已被其他用户绑定的邮箱
        another_user = BaseInfo.objects.create(
            username="anotheruser",
            fullname="Another User",
            email="another@example.com",
            phone="1111111111",
            uuid=JULIDField.get_u12()
        )
        params = {
            "email": "another@example.com"
        }
        result, err = UserService.user_edit(params=params, user_id=self.user.id)
        self.assertIsNotNone(err)
        self.assertEqual(err, "当前邮箱被绑定")

    def test_user_edit_with_user_uuid(self):
        # 测试通过 user_uuid 更新用户信息
        params = {
            "fullname": "Updated User via UUID",
            "nickname": "updatednick_via_uuid",
            "email": "updated_via_uuid@example.com",
            "phone": "0987654321",
            "is_using": False
        }
        result, err = UserService.user_edit(params=params, user_uuid=self.user.uuid)
        self.assertEqual(err, None)
        self.assertEqual(result["fullname"], "Updated User via UUID")
        self.assertEqual(result["nickname"], "updatednick_via_uuid")
        self.assertEqual(result["email"], "updated_via_uuid@example.com")
        self.assertEqual(result["phone"], "0987654321")
        self.assertEqual(result["is_using"], False)


# encoding: utf-8
"""
@project: djangoModel->sync_table_structure
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 更新表结构
@created_time: 2023/8/22 9:09
"""
from django.core.management import BaseCommand


class Command(BaseCommand):
    # 帮助文本, 一般备注命令的用途及如何使用。
    help = "测试调试结构"

    # 给命令添加一个名为name的参数
    def add_arguments(self, parser):
        pass

    # 核心业务逻辑，通过options字典接收name参数值，拼接字符串后输出
    def handle(self, *args, **options):
        from xj_user.services.user_apply_services import UserApplyServices
        data, err = UserApplyServices.add_apply_record(params={
            "apply_type_id": 1,
            "user_id": 1,
            "remark": "加急备注",
            "verify_files": {"id_card_front": "https://baidu.com"}
        })
        print(data, err)



# from django.conf import settings
from django.contrib import admin

# 引入用户平台
from .models import *


# admin.site.site_header = settings.MAIN_CONFIG.get('site_header')
# admin.site.site_title = settings.MAIN_CONFIG.get('site_title')


class BaseInfoAdmin(admin.ModelAdmin):
    # fields = ('id', 'uuid', 'user_no', 'username', 'nickname', 'fullname', 'phone', 'email', 'user_type', 'privacies',
    #           'user_info', 'register_ip', 'register_time', 'is_delete')
    fieldsets = [
        ('用户基础信息', {'classes': 'collapse', 'fields': (
            ('id', 'uuid'), 'user_no', ('username', 'nickname'), 'fullname', ('phone', 'email'), 'user_type',
            ('user_info', 'privacies'), ('register_time', 'register_ip'), 'is_delete'
        )})
    ]
    list_display = (
        'id', 'uuid_short', 'user_no', 'username', 'nickname', 'fullname', 'phone', 'email', 'user_type',
        'user_info_short', 'privacies_short', 'register_ip', 'register_time', 'is_delete')
    list_display_links = ['username', 'phone', 'email']
    list_filter = ['user_type']
    search_fields = ('uuid', 'username', 'fullname', 'nickname', 'email', 'phone')
    readonly_fields = ['id', 'uuid']
    list_per_page = 20


class DetailInfoAdmin(admin.ModelAdmin):
    # fields = (
    #     'id', 'user', 'real_name', 'sex', 'birth', 'tags', 'signature', 'avatar', 'cover', 'id_card_type', 'id_card_no',
    #     'language', 'region_code', 'more',
    #     'field_1', 'field_2', 'field_3', 'field_4', 'field_5', 'field_6', 'field_7', 'field_8', 'field_9',
    #     'field_10', 'field_11', 'field_12', 'field_13', 'field_14', 'field_15'
    # )

    fieldsets = [
        ('用户详细信息', {'classes': 'collapse', 'fields': (
            'id', ('user', 'real_name'), ('sex', 'birth'), ('tags', 'signature'), ('avatar', 'cover'),
            ('id_card_type', 'id_card_no'), ('language', 'region_code'), 'more',
            ('field_1', 'field_2'), ('field_3', 'field_4'), ('field_5', 'field_6'), ('field_7', 'field_8'),
            ('field_9', 'field_10'), ('field_11', 'field_12'), ('field_13', 'field_14'), ('field_15')
        )})
    ]
    list_display = ('id', 'user_short', 'real_name', 'sex', 'birth', 'tags', 'avatar', 'cover', 'language', 'region_code',
                    )
    list_display_links = ['user_short']
    search_fields = ('user__username', 'user__fullname', 'real_name')
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class AuthAdmin(admin.ModelAdmin):
    fields = ('id', 'platform', 'user', 'password', 'salt', 'algorithm', 'token', 'ticket',
              'last_update_ip', 'create_time', 'update_time')
    list_display = ('platform', 'user', 'password', 'salt', 'algorithm', 'token_short',
                    'ticket_short', 'create_time', 'update_time')
    list_display_links = ['user']
    search_fields = ('user__username', 'user__fullname')
    list_filter = ['platform']
    readonly_fields = ['id', 'create_time', 'update_time']
    raw_id_fields = ['user']
    list_per_page = 20

    # def platform(self, obj):
    #     return obj.platform


class ExtendFieldAdmin(admin.ModelAdmin):
    fields = ('id', 'field', 'field_index', 'description', 'type', 'config', 'default', 'sort')
    list_display = ('field', 'field_index', 'description', 'type', 'config', 'default', 'sort')
    readonly_fields = ['id']
    list_per_page = 20


class AccessLogAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'ip', 'create_time', 'client_info', 'more',)
    list_display = ('user', 'ip', 'create_time', 'client_info',)
    readonly_fields = ['id', 'create_time']
    raw_id_fields = ['user']
    list_per_page = 20


class HistoryAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'field', 'old_value', 'new_value', 'create_time',)
    list_display = ('user', 'field', 'old_value', 'new_value', 'create_time',)
    readonly_fields = ['id', 'create_time']
    raw_id_fields = ['user']
    list_per_page = 20


class RestrictRegionAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'region_code',)
    list_display = ('user', 'region_code',)
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class PlatformAdmin(admin.ModelAdmin):
    fields = ('id', 'platform_id', 'platform_code', 'platform_name', 'root_category_value')
    list_display = ('id', 'platform_id', 'platform_code', 'platform_name', 'root_category_value')
    list_display_links = ['platform_code']
    search_fields = ('platform_id', 'platform_name', 'platform_code')
    list_per_page = 20


class PlatformsToUsersAdmin(admin.ModelAdmin):
    fields = ('user', 'platform', 'platform_user_id',)
    list_display = ('platform', 'user', 'platform_user_id',)
    list_filter = ['platform']
    # readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class ContactBookAdmin(admin.ModelAdmin):
    fields = ('id', 'user', 'friend', 'phone', 'phones', 'telephone', 'telephones',
              'email', 'qq', 'address', 'more', 'remarks')
    list_display = ('id', 'user', 'friend', 'phone', 'phones', 'telephone', 'telephones',
                    'email', 'qq', 'address', 'more', 'remarks')
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class UserSsoServeAdmin(admin.ModelAdmin):
    fields = ('id', 'sso_code', 'sso_name', 'sso_url', 'description', 'sso_appid', 'sso_account_id')
    list_display = ('id', 'sso_code', 'sso_name', 'sso_url', 'description')
    readonly_fields = ['id']
    list_per_page = 20


class UserSsoToUserAdmin(admin.ModelAdmin):
    fields = ('id', 'sso_serve', 'user', 'sso_unicode', 'sso_ticket', 'union_code',)
    list_display = ('id', 'sso_serve', 'user', 'sso_unicode', 'sso_ticket', 'union_code')
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class UserRelateTypeAdmin(admin.ModelAdmin):
    fields = ("id", "relate_key", "relate_name", "is_multipeople", "description",)
    list_display = ("id", "relate_key", "relate_name", "is_multipeople", "description",)
    readonly_fields = ['id']
    list_per_page = 20


class UserRelateToUserAdmin(admin.ModelAdmin):
    fields = ("id", "user", "with_user", "user_relate_type",)
    list_display = ("id", "user", "with_user", "user_relate_type",)
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


class UserBankCardsAdmin(admin.ModelAdmin):
    fields = ("id", "user", "bank_card_num", "open_account_bank")
    list_display = ("id", "user", "bank_card_num", "open_account_bank")
    readonly_fields = ['id']
    raw_id_fields = ['user']
    list_per_page = 20


admin.site.register(BaseInfo, BaseInfoAdmin)
admin.site.register(DetailInfo, DetailInfoAdmin)
admin.site.register(Auth, AuthAdmin)
admin.site.register(ExtendField, ExtendFieldAdmin)
# admin.site.register(AccessLog, AccessLogAdmin)
# admin.site.register(History, HistoryAdmin)
# admin.site.register(RestrictRegion, RestrictRegionAdmin)
admin.site.register(Platform, PlatformAdmin)
admin.site.register(PlatformsToUsers, PlatformsToUsersAdmin)
# admin.site.register(Permission, PermissionAdmin)
# admin.site.register(PermissionValue, PermissionValueAdmin)
# admin.site.register(Group, GroupAdmin)
# admin.site.register(ContactBook, ContactBookAdmin)
admin.site.register(UserSsoServe, UserSsoServeAdmin)
admin.site.register(UserSsoToUser, UserSsoToUserAdmin)
# admin.site.register(UserRelateType, UserRelateTypeAdmin)
# admin.site.register(UserRelateToUser, UserRelateToUserAdmin)
admin.site.register(UserBankCards, UserBankCardsAdmin)



# coding: utf-8
import inspect

from django.core.paginator import Paginator
from django.core.cache import cache
from django.utils.functional import cached_property
from django.utils.inspect import method_has_no_args


class CustomPaginator(Paginator):

    @cached_property
    def count(self):
        """
        重写Paginator类的count方法，并将结果数据缓存到数据库中
        """
        # print('> CustomPaginator: count!', self)
        table_name = self.object_list.model._meta.db_table
        key_name = f'table_{table_name}__count'
        # print('> CustomPaginator: key_name:', key_name)
        n = cache.get(key_name, default=None)  # 从缓存中读数据表总行数
        if n is not None:
            return n

        # 调整后的原始代码逻辑，重新计划数据表总行数
        fn = getattr(self.object_list, 'count', None)
        n = fn() if callable(fn) and not inspect.isbuiltin(fn) and method_has_no_args(fn) else len(self.object_list)
        # print('> CustomPaginator: n:', n)

        cache.set(key_name, n, timeout=86400)  # 写缓存，数据表总行数
        return n



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
from rest_framework.request import Request

from utils.custom_tool import force_transform_type, write_to_log, parse_json
from xj_user.utils.dynamic_load_class import dynamic_load_class


# 流程调用装饰器
def flow_service_wrapper(func):
    """
    API 流程中间件装饰器
    PS 该装饰器必须配套request_params_wrapper 和 user_wrapper 一起使用
    PS 开放的服务必须使用key_value类型接收参数，如：params: dict = None, **kwargs
    """

    # 解析请求参数 兼容 APIView与View的情况，View 没有request.data
    def wrapper(instance, arg_request=None, *args, request=None, request_params=None, user_info: dict = None, **kwargs):
        """
        @param instance 实例是一个APIView的实例
        @param args 其它可变参数元组
        @param kwargs 其它可变关键字参数字典
        :param user_info: 用户信息，使用用户登录验证装饰器才会生效。
        :param request_params: 请求参数解析
        """
        if isinstance(instance, WSGIRequest) or isinstance(instance, Request) or isinstance(instance, ASGIRequest):
            request = instance
        if isinstance(arg_request, WSGIRequest) or isinstance(arg_request, Request) or isinstance(arg_request, ASGIRequest):
            request = arg_request

        # ----------------- section 得到请求参数以及用户信息 start --------------------------
        user_info, is_pass = force_transform_type(variable=user_info, var_type="dict", default={})
        request_params, is_pass = force_transform_type(variable=request_params, var_type="dict", default={})
        # 自动补全，用户信息字段
        for k, v in user_info.items():
            request_params.setdefault(k, v)
        # ----------------- section 得到请求参数以及用户信息 end   --------------------------

        # ----------------- section 加载流程类,并判断是否要执行流程 start --------------------------
        CopyFlowProcessService, is_import = dynamic_load_class(import_path="xj_flow.services.flow_process_service", class_name="FlowProcessService")
        # 检查请求参数中是否由流程相关信息,判断是触发流程
        flow_node_id = request_params.pop("flow_node_id", None)
        flow_action_id = request_params.pop("flow_action_id", None)
        flow_node_value = request_params.pop("flow_node_value", None)
        flow_action_value = request_params.pop("flow_action_value", None)

        if is_import or (not flow_node_id and not flow_node_value) or (not flow_action_id and not flow_action_value):
            return func(instance, *args, request=request, request_params=request_params, user_info=user_info.copy(), **kwargs)
        # ----------------- section 加载流程类,并判断是否要执行流程 end   --------------------------

        service = CopyFlowProcessService()
        # ----------------- section 流程阻断判断 start --------------------------
        data, is_prevent = service.flow_switch(flow_node_id=flow_node_id, flow_node_value=flow_node_value)
        if is_prevent:
            from .custom_response import util_response
            return util_response(err=5000, msg=is_prevent)
        # ----------------- section 流程阻断判断 end   --------------------------

        # ----------------- section 执行前置流程方法 start --------------------------
        data, err = service.do_once_flow_in_service(
            flow_node_id=flow_node_id,
            flow_node_value=flow_node_value,
            flow_action_id=flow_action_id,
            flow_action_value=flow_action_value,
            source_params=request_params.copy(),
            run_mode="BEFORE"
        )
        # 如果有错误则计入执行错误日志
        if err:
            write_to_log(prefix="流程装饰器调用异常:", content=err)
        # 处理后的参数默认值补全
        data = data or {}
        request_params = data.get("source_params", request_params)
        # ----------------- section 执行前置流程方法 end   --------------------------

        # ----------------- section 执行接口方法 start --------------------------
        response = func(instance, *args, request=request, request_params=request_params.copy(), user_info=user_info.copy(), **kwargs)
        # 获取http响应的内容
        try:
            response_json = parse_json(response.content.decode(encoding="utf-8"))
            response_json, is_pass = force_transform_type(variable=response_json, var_type="dict", default={})
            response_err = response_json.detail("err", None)
            response_data, is_pass = force_transform_type(variable=response_json.detail("data", {}), var_type="only_dict", default={})
        except ValueError:
            response_json = {}
            response_err = None
            response_data = {}
        # ----------------- section 执行接口方法 end   --------------------------

        # ----------------- section 执行后置流程方法 start --------------------------
        if not response_err:
            # 如果请求接口没有报错则不可以执行
            request_params.update(response_data)
            data, err = service.do_once_flow_in_service(
                flow_node_id=flow_node_id,
                flow_node_value=flow_node_value,
                flow_action_id=flow_action_id,
                flow_action_value=flow_action_value,
                source_params=request_params.copy(),
                run_mode="AFTER"
            )
            if err:
                write_to_log(prefix="流程装饰器调用异常:", content=err)
        # ----------------- section 执行后置流程方法 end   --------------------------

        # ----------------- section 记录流程记录 start --------------------------
        # 流程完成记录
        if not response_err:
            data, flow_err = service.finish_flow(user_id=user_info.detail("user_id"))
            if flow_err:
                write_to_log(prefix="流程装饰器保存流程记录失败", content=flow_err)
        # 规则执行记录
        record, record_err = service.save_record(result_dict=response_json.get("data", {}), user_info=user_info.copy())
        if record_err:
            write_to_log(prefix="流程装饰器保存流程记录失败", content=record_err)
        # ----------------- section 记录流程记录 end   --------------------------
        return response

    return wrapper



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
import copy
import importlib
import sys


def dynamic_load_class(import_path: str = None, class_name: str = None, find_services=False):
    """
    动态加载模块中的类,返回类的指针
    可从通过RPC协议从consul服务获取其他服务器的服务类。
    :param find_services: 是否使用RPC，发现服务
    :param import_path: 导入类的文件路径
    :param class_name: 导入文件中的哪一个类
    :return: class_instance,err_msg
    """
    try:
        class_instance = getattr(sys.modules.get(import_path), class_name, None)
        if class_instance is None:
            import_module = importlib.import_module(import_path)
            class_instance = getattr(import_module, class_name)
        copy_class_instance = copy.deepcopy(class_instance)
        return copy_class_instance, None
    except AttributeError:
        return None, "系统中不存在该模块"
    except Exception as e:
        return None, str(e)



from django.test import TestCase
from ..services.user_service import UserService
from ..models import BaseInfo
from ..utils.j_ulid_field import JULIDField

class TestUserSearch(TestCase):
    def setUp(self):
        # Create 10 users for testing
        self.users = []
        for i in range(10):
            user = BaseInfo.objects.create(
                username=f"testuser{i}",
                fullname=f"Test User {i}",
                nickname=f"testnick{i}",
                email=f"test{i}@example.com",
                phone=f"123456789{i}",
                user_info="{}",
                privacies="{}",
                is_using=True,
                uuid=JULIDField.get_u12()
            )
            self.users.append(user)

    def test_user_search_by_id_list(self):
        # Test searching users by a list of user IDs
        user_id_list = [user.id for user in self.users[:5]]  # Search for the first 5 users
        result, err = UserService.user_search(user_id_list=user_id_list)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 5)  # Expect 5 users in the result

    def test_user_search_by_search_params(self):
        # Test searching users by search parameters
        search_params = {"username": "testuser1"}
        result, err = UserService.user_search(search_params=search_params)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 1)  # Expect 1 user in the result
        self.assertEqual(result["data"][0]["username"], "testuser1")

    def test_user_search_by_email(self):
        # Test searching users by email
        search_params = {"email": "test1@example.com"}
        result, err = UserService.user_search(search_params=search_params)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 1)  # Expect 1 user in the result
        self.assertEqual(result["data"][0]["email"], "test1@example.com")

    def test_user_search_by_fullname(self):
        # Test searching users by fullname
        search_params = {"fullname": "Test User 2"}
        result, err = UserService.user_search(search_params=search_params)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 1)  # Expect 1 user in the result
        self.assertEqual(result["data"][0]["fullname"], "Test User 2")

    def test_user_search_pagination(self):
        # Test searching users with pagination
        search_params = {}
        result, err = UserService.user_search(search_params=search_params, page=1, page_size=5)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 5)  # Expect 5 users in the result
        self.assertEqual(result["pagination"]["total"], 10)  # Total users should be 10
        self.assertEqual(result["pagination"]["page"], 1)  # Current page should be 1
        self.assertEqual(result["pagination"]["page_size"], 5)  # Page size should be 5

    def test_user_search_order_by_username(self):
        # Test searching users with ordering by username
        search_params = {}
        result, err = UserService.user_search(search_params=search_params, order_by="username")
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 10)  # Expect all 10 users in the result
        # Check if the users are ordered by username
        usernames = [user["username"] for user in result["data"]]
        self.assertEqual(usernames, sorted(usernames))

    def test_user_search_order_by_fullname_desc(self):
        # Test searching users with ordering by fullname in descending order
        search_params = {}
        result, err = UserService.user_search(search_params=search_params, order_by="-fullname")
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 10)  # Expect all 10 users in the result
        # Check if the users are ordered by fullname in descending order
        fullnames = [user["fullname"] for user in result["data"]]
        self.assertEqual(fullnames, sorted(fullnames, reverse=True))

    def test_user_search_with_filter_fields(self):
        # Test searching users with specific filter fields
        search_params = {}
        filter_fields = ["username", "email"]
        result, err = UserService.user_search(search_params=search_params, filter_fields=filter_fields)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 10)  # Expect all 10 users in the result
        # Check if only the specified fields are returned
        for user in result["data"]:
            self.assertIn("username", user)
            self.assertIn("email", user)
            self.assertNotIn("fullname", user)

    def test_user_search_with_need_map(self):
        # Test searching users with need_map=True
        search_params = {}
        result, err = UserService.user_search(search_params=search_params, need_map=True)
        self.assertEqual(err, None)
        self.assertEqual(len(result["data"]), 10)  # Expect all 10 users in the result
        # Check if the result is a dictionary with user_id as keys
        self.assertIsInstance(result["data"], dict)
        for user_id, user_info in result["data"].items():
            self.assertIn("user_id", user_info)
            self.assertEqual(user_id, str(user_info["user_id"]))

    def test_user_search_with_invalid_page(self):
        # Test searching users with an invalid page number
        search_params = {}
        result, err = UserService.user_search(search_params=search_params, page=100, page_size=5)
        self.assertEqual(err, "页码超出范围")
        self.assertEqual(result, [])  # Expect an empty result


from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.serialization import load_pem_public_key, load_pem_private_key
import base64


def encrypt_password(pwd, dypub_pem, salt):
    # 将动态公钥从PEM格式加载为RSA公钥对象
    dypub = load_pem_public_key(dypub_pem.encode())

    # 生成密钥派生函数（KDF）的参数
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt.encode(),
        iterations=100000,
    )

    # 使用KDF和密码生成加密密钥
    key = kdf.derive(pwd.encode())

    # 使用动态公钥加密密钥
    encrypted_key = dypub.encrypt(
        key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )

    # 将加密密钥编码为Base64，以便于传输
    cip = base64.b64encode(encrypted_key).decode()

    return cip


# 示例用法
# pwd = "user_password"
# dypub_pem = """
# -----BEGIN PUBLIC KEY-----
# ... your public key in PEM format ...
# -----END PUBLIC KEY-----
# """
# salt = "a1b2c3"
#
# cip = encrypt_password(pwd, dypub_pem, salt)
# print("传输密文 (cip):", cip)


def decrypt_password(cip, privkey_pem, salt, passphrase=None):
    # 将私钥从PEM格式加载为RSA私钥对象
    privkey = load_pem_private_key(privkey_pem.encode(), password=passphrase, backend=None)

    # 解码Base64编码的密文，以获取加密密钥
    encrypted_key = base64.b64decode(cip)

    # 使用私钥解密密钥
    key = privkey.decrypt(
        encrypted_key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None
        )
    )

    # 生成密钥派生函数（KDF）的参数
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt.encode(),
        iterations=100000,
    )

    # 验证解密后的密钥与原始密码是否匹配
    try:
        kdf.verify(key, key)
        print("密码匹配")
    except Exception:
        print("密码不匹配")


# 示例用法
# cip = "your_base64_encoded_encrypted_key"
# privkey_pem = """
# -----BEGIN PRIVATE KEY-----
# ... your private key in PEM format ...
# -----END PRIVATE KEY-----
# """
# salt = "a1b2c3"
# passphrase = None  # 如果您的私钥有密码，请提供
#
# decrypt_password(cip, privkey_pem, salt, passphrase)



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
from cmath import cos
import copy
import datetime
import difflib
import importlib
import inspect
import json
from logging import getLogger
from math import sin, asin, sqrt
import random
import re
import sys
import time
from urllib.parse import parse_qs
import uuid

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
# from numpy.core._multiarray_umath import radians
from rest_framework.request import Request
# import xmltodict


def filter_fields_handler(
        default_field_list: list = None,
        input_field_expression: 'str|list' = None,
        split_char: str = ";",
        all_field_list: list = None
) -> list:
    """
    过滤字段处理器
    使用：服务提供者只需要提供一个默认字段的列表或者符号分割的字符串，然后再把前端传进来的字段表达式传进来即可
    :param all_field_list: 全部合法性字段
    :param default_field_list: 默认字段列表,或者是符号分割的字符串
    :param input_field_expression: 字段处理表达式。"***filed_1;filed_2;;filed_2;!!!filed_1;filed_2;" 或者"filed_1;filed_2"或者 [filed_1,filed_2;]
    :param split_char: 拆分字符串,默认使用分号。
    :return: ["field_1",.....]
    """
    # all_field_list 与 default_field_list 因为是在服务层调用，所以强制列表类型
    if all_field_list is None or not isinstance(all_field_list, list):
        all_field_list = []
    if default_field_list is None or not isinstance(default_field_list, list):
        default_field_list = []
    # 处理默认字段
    default_field_list = default_field_list.split(split_char) if isinstance(default_field_list, str) else default_field_list or all_field_list

    # ========== 处理字段处理表达式 ==========
    # 如果没有传递字段表达式，默认字段不为空。则返回默认字段
    if input_field_expression is None:
        return default_field_list

    # 字段表达式字符串的情况
    elif isinstance(input_field_expression, str):
        if not re.search("[***|!!!]", input_field_expression):
            return format_list_handle(param_list=input_field_expression.split(split_char), filter_filed_list=all_field_list)
        # 加法或者减法原则
        default_field_hash = {i: "" for i in default_field_list}
        add_filed_expression = re.search("[***][^(!!!)]*", input_field_expression)
        sub_filed_expression = re.search("[!!!][^(***)]*", input_field_expression)
        if add_filed_expression:
            add_filed_list = add_filed_expression.group().replace("***", "").split(split_char)
            for i in add_filed_list:
                default_field_hash.update({i: ""})
        if sub_filed_expression:
            sub_filed_list = sub_filed_expression.group().replace("!!!", "").split(split_char)
            for i in sub_filed_list:
                default_field_hash.pop(i, None)
        return format_list_handle(param_list=[i for i in list(default_field_hash.keys()) if i], filter_filed_list=all_field_list)

    # 如果是列表则代表用户使用自定义的字段列表，不使用默认的字段列表
    elif isinstance(input_field_expression, list):
        return format_list_handle(param_list=input_field_expression, filter_filed_list=all_field_list)

    # input_field_expression的类型都不符合，则返回default_field_list
    else:
        return default_field_list
    # ========== 处理字段处理表达式 ==========



# _*_coding:utf-8_*_

from django.urls import re_path

from .apis import user_apply_apis
from .apis import user_cards_apis
# from .apis import user_contact_book
from .apis import user_detail_info
from .apis import user_edit
from .apis import user_info
from .apis import user_list
from .apis import user_login
# TODO 用户模块不希望依赖xj_captcha模块 20241206 by Sieyoo
from .apis import user_login_short_message
from .apis import user_login_wechat
from .apis import user_password
from .apis import user_platform
from .apis import user_relate_apis
from .apis import user_sso_serve
from .apis import user_statistics
from .apis.user_add import UserAdd
from .apis.user_login_main import UserLoginMain
# from .service_register import register
# from .apis import user_subcompany_apis

# 应用名称
app_name = 'xj_user'

# register()
# 应用路由
urlpatterns = [
    # 平台相关API
    re_path(r'^platform/?$', user_platform.UserPlatform.as_view(), name="用户的平台CRUD"),
    re_path(r'^platform_list/?$', user_platform.UserPlatform.list, name="用户的平台列表"),
    re_path(r'^get_user_platform/?$', user_platform.UserPlatform.get_user_platform, name="获取当前用户的平台信息"),

    # 登录操作
    re_path(r'^login/?$', user_login.UserLogin.as_view(), name="用户密码登录"),
    re_path(r'^login_main/?$', UserLoginMain.login_main, name="登录总接口"),
    re_path(r'^js_sdk/?$', user_sso_serve.UserSsoServe.get_js_sdk, name="微信公众号JS-SDK"),
    # TODO 用户模块不希望依赖xj_captcha模块 20241206 by Sieyoo
    # re_path(r'^login_short_message/?$', user_login_short_message.ShortMessageLogin.sms_login, name="短信登录"),
    re_path(r'^sso_serve/?$', user_sso_serve.UserSsoServe.as_view(), ),
    re_path(r'^login_wechat/?$', user_login_wechat.WechetLogin.as_view(), name="微信登录"),
    re_path(r'^register/?$', UserLoginMain.register, name="用户注册"),
    re_path(r'^add/?$', UserAdd.add, name="管理员添加用户"),
    re_path(r'^secondary_authorization/?$', UserLoginMain.secondary_authorization, name="二次授权"),
    re_path(r'^bind_phone/?$', UserLoginMain.bind_phone, name="绑定手机号"),
    re_path(r'^login_phone/?$', user_login_short_message.ShortMessageLogin.sms_login, name="手机验证码登录"),


    # 个人信息相关
    re_path(r'^info/?$', user_info.UserInfo.as_view(), name="查看个人信息"),
    re_path(r'^edit/?$', user_edit.UserEdit.user_edit, name="user_list_detail用户信息编辑"),
    re_path(r'^delete/?$', user_edit.UserEdit.as_view(), name="用户删除"),
    re_path(r'^password/?$', user_password.UserPassword.as_view(), name="用户修改验证码"),
    re_path(r'^list/?$', user_list.UserListAPIView.as_view(), name="用户列表"),
    re_path(r'^bind_identity/?$', user_edit.UserEdit.bind_identity, name="用户身份绑定"),

    # 详细信息以及绑定信息
    re_path(r'^detail/?$', user_detail_info.UserDetail.as_view(), name="查看用户的详细信息"),
    re_path(r'^list_detail/?$', user_detail_info.UserListDetail.as_view(), name="用户详细信息列表"),
    re_path(r'^detail_edit/?$', user_detail_info.UserDetailEdit.as_view(), name="用户详细信息编辑"),
    re_path(r'^extend_fields/?$', user_detail_info.UserDetailExtendFields.as_view(), name="用户详细信息扩展字段映射"),
    re_path(r'^bank_card/?(?P<detail_id>\d+)?$', user_cards_apis.UserBankAPIView.as_view(), name="银行卡管理CRUD"),
    # re_path(r'^contact_book/?$', user_contact_book.UserContactBook.as_view(), name="用户通讯录"),

    # 用户关系-以及相关的营销或等接口
    re_path(r'^relate_type/?$', user_relate_apis.UserRelateTypeApis.as_view(), name="关系类型CURD"),
    re_path(r'^relate_user/?$', user_relate_apis.UserRelateToUserApis.as_view(), name="用户与关系关系类型映射的CRUD"),
    re_path(r'^distribution_relate/?$', user_relate_apis.UserRelateToUserApis.distribution_relate, name="用戶分三級分銷關係查詢"),

    # 镖行业务接口
    re_path(r'^bind_bxtx_relate/?$', user_relate_apis.UserRelateToUserApis.bind_bxtx_relate, name="镖行业务接口-绑定业务员和邀请人"),
    re_path(r'^bxtx_relate_user/?$', user_relate_apis.UserRelateToUserApis.bxtx_relate_user, name="镖行业务接口-查看下级用户"),

    # 劳务通业务接口
    re_path(r'^relate_user_tree/?$', user_relate_apis.UserRelateToUserApis.relate_user_tree, name="劳务通-树状结构接口"),
    re_path(r'^laowu_bind_bxtx_relate/?$', user_relate_apis.UserRelateToUserApis.laowu_bind_bxtx_relate, name="劳务通-绑定关系"),

    # 待分离接口
    re_path(r'^statistics/?$', user_statistics.UserStatisticsAPI.as_view(), ),


    # 用户审批接口
    re_path(r'^apply_type_list/?$', user_apply_apis.UserApplyApis.apply_type_list, name="审配类型列表"),
    re_path(r'^apply_add/?$', user_apply_apis.UserApplyApis.add_user_apply, name="提交用户审批"),
    re_path(r'^agree_apply/?$', user_apply_apis.UserApplyApis.agree_user_apply, name="同意用户审批"),
    re_path(r'^disagree_apply/?$', user_apply_apis.UserApplyApis.disagree_user_apply, name="同意用户审批"),
    re_path(r'^apply_list/?$', user_apply_apis.UserApplyApis.user_apply_list, name="用户审批列表"),
    re_path(r'^apply_statistics/?$', user_apply_apis.UserApplyApis.user_apply_statistics, name="用户审批统计"),

    # 分子公司接口
    # re_path(r'^subcompany/?(?P<detail_id>\d+)?$', user_subcompany_apis.UserSubCompanyAPIView.as_view(), name="分子公司管理"),

]



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
from cmath import cos
import copy
import datetime
import difflib
import importlib
import inspect
import json
from logging import getLogger
from math import sin, asin, sqrt
import random
import re
import sys
import time
from urllib.parse import parse_qs
import uuid

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
# from numpy.core._multiarray_umath import radians
from rest_framework.request import Request
# import xmltodict


# 写入日志
def write_to_log(level="info", prefix="系统异常", content="", err_obj=None):
    """
    写入日志, 注意仅仅支持python3.0以上版本
    :param level: 写入错误日志等级
    :param prefix: 提示错误类型
    :param content: 错误内容
    :param err_obj: try except 捕捉到的错误对象
    :return: data, err_msg
    """
    logger = getLogger('log')
    try:
        if not err_obj is None:
            logger.error(
                '---' + prefix + ":" + str(err_obj) + ";" +
                (" content:" + str(content) + ";" if content else "") +
                " line:" + str(err_obj.__traceback__.tb_lineno) + ";" +
                " file:" + str(err_obj.__traceback__.tb_frame.f_globals["__file__"]) + ";" +
                " datetime:" + time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time.time())) + ";"
            )
        elif level == "info":
            logger.error('---' + prefix + ":" + str(content))
        elif level == "error":
            logger.error('---' + prefix + ":" + str(content))
        return True, None
    except Exception as err:
        return False, str(err)



# -*- coding: utf-8 -*-

import os
import itertools
import unicodedata

from ._compat import u

__all__ = ['get', 'get_pinyin', 'get_initial']


# init pinyin dict
pinyin_dict = {}
pinyin_tone = {}
dat = os.path.join(os.path.dirname(__file__), "Mandarin.dat")
with open(dat) as f:
    for line in f:
        k, v = line.strip().split('\t')
        pinyin_dict[k] = u(v.lower().split(" ")[0][:-1])
        pinyin_tone[k] = int(v.lower().split(" ")[0][-1])


def _pinyin_generator(chars, format):
    """Generate pinyin for chars, if char is not chinese character,
    itself will be returned.
    Chars must be unicode list.
    """
    for char in chars:
        key = "%X" % ord(char)
        pinyin = pinyin_dict.get(key, char)
        tone = pinyin_tone.get(key, 0)

        if tone == 0 or format == "strip":
            pass
        elif format == "numerical":
            pinyin += str(tone)
        elif format == "diacritical":
            # Find first vowel -- where we should put the diacritical mark
            vowels = itertools.chain((c for c in pinyin if c in "aeo"),
                                     (c for c in pinyin if c in "iuv"))
            vowel = pinyin.index(next(vowels)) + 1
            pinyin = pinyin[:vowel] + tonemarks[tone] + pinyin[vowel:]
        else:
            error = "Format must be one of: numerical/diacritical/strip"
            raise ValueError(error)

        yield unicodedata.normalize('NFC', pinyin)


def get(s, delimiter='', format="diacritical"):
    """Return pinyin of string, the string must be unicode
    """
    return delimiter.join(_pinyin_generator(u(s), format=format))


def get_pinyin(s):
    """This function is only for backward compatibility, use `get` instead.
    """
    import warnings
    warnings.warn('Deprecated, use `get` instead.')
    return get(s)


def get_initial(s, delimiter=' '):
    """Return the 1st char of pinyin of string, the string must be unicode
    """
    initials = (p[0] for p in _pinyin_generator(u(s), format="strip"))
    return delimiter.join(initials)

tonemarks = ["", u("̄"), u("́"), u("̌"), u("̀"), ""]



# encoding: utf-8
"""
@project: djangoModel->tool
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 常用工具封装，同步所有子模块新方法，保持公共模块的工具库最新版本
@created_time: 2022/6/15 14:14
"""
from cmath import cos
import copy
import datetime
import difflib
import importlib
import inspect
import json
from logging import getLogger
from math import sin, asin, sqrt
import random
import re
import sys
import time
from urllib.parse import parse_qs
import uuid

from django.core.handlers.asgi import ASGIRequest
from django.core.handlers.wsgi import WSGIRequest
# from numpy.core._multiarray_umath import radians
from rest_framework.request import Request
# import xmltodict

# =========== section 单例装饰器 start ==================


_instance = {}


# 过滤list内容，白名单、黑名单、别名
def format_list_handle(param_list, filter_filed_list=None, remove_filed_list=None, alias_dict=None, remove_repeat=True):
    """
    过滤list内容
    :param param_list: 传入 param_list
    :param filter_filed_list: 需要的字段
    :param remove_filed_list: 需要删除的列表
    :param alias_dict: 元素起别名
    :return:param_list： 处理后的 param_list
    """
    if not param_list:
        return param_list
    # 类型判断 过滤字段
    if filter_filed_list and isinstance(filter_filed_list, list):
        param_list = [i for i in param_list if i in filter_filed_list]

    # 类型判断， 剔除字段
    if remove_filed_list and isinstance(remove_filed_list, list):
        param_list = [j for j in param_list if not j in remove_filed_list]

    # 类型判断 字段转换
    if alias_dict and isinstance(alias_dict, dict):
        param_list = [alias_dict.get(k, k) for k in param_list]

    # 进行去重
    if remove_repeat:
        param_list = list(set(param_list))

    return param_list



# encoding: utf-8
"""
@project: djangoModel->user_info_service
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 用户信息服务
@created_time: 2022/6/27 19:51
"""
from django.core.paginator import Paginator, EmptyPage
from django.db.models import F, Case, When, Value, CharField, OuterRef, Subquery
from django.db.models import Prefetch
from django.db.models.functions import Coalesce
from django.forms import model_to_dict

from xj_user.utils.j_parameter import JParameter
# from orator import DatabaseManager

# from config.config import JConfig
from ..models import BaseInfo, DetailInfo, ExtendField
# from ..models import UserApplyRecord
from ..services.user_service import UserService
from ..utils.j_transform_type import JTransformType
from ..utils.format_params_handle import format_params_handle
from ..utils.filter_fields_handler import filter_fields_handler
from ..utils.format_list_handle import format_list_handle
from ..utils.dynamic_load_class import dynamic_load_class


# from ..utils.custom_tool import *

# orator ORM 初始化
# config = JConfig()
# db_config = {
#     config.get('main', 'driver', "mysql"): {
#         'driver': config.get('main', 'driver', "mysql"),
#         'host': config.get('main', 'mysql_host', "127.0.0.1"),
#         'database': config.get('main', 'mysql_database', ""),
#         'user': config.get('main', 'mysql_user', "root"),
#         'password': config.get('main', 'mysql_password', "123456"),
#         "port": config.getint('main', 'mysql_port', "3306")
#     }
# }
# db = DatabaseManager(db_config)


class DetailInfoService:
    # 用户基础字段
    base_fields = UserService.base_fields
    # 用户详情字段。不含扩展
    detail_fields = ['real_name', 'sex', 'birth', 'tags', 'signature', 'cover', 'id_card_type', 'id_card_no',
                     'language', 'region_code', 'more']

    # user_detail_expect_extend = [i.name for i in DetailInfo._meta.fields if not "field_" in i.name] + ["user_id"]
    # user_detail_remove_fields = [i.name for i in DetailInfo._meta.fields if "field_" in i.name] + ["id", "user"]
    # 详情信息扩展字段获取。该行代码在执行manage.py makemigragtion 会报表不存在 20230822 by Sieyoo
    # field_map_list = list(ExtendField.objects.all().values("field", 'field_index'))
    # field_map_list = list(ExtendField.objects.all().values("field", 'field_index')) if ExtendField.objects.all().values(
    #     "field", 'field_index') else list()
    field_map_list = []

    @classmethod
    def get_list_detail(cls, params: dict = None, user_id_list: list = None, filter_fields: list = None, **kwargs):
        """
        详细信息列表
        :param filter_fields: 需要过滤的字段
        :param params: 搜索参数
        :param user_id_list: 用户ID列表。请注意：如果传递过来的参数不是有效的列表，则按照正常的分页搜索返回。
        :return: list,err
        """
        params, is_pass = force_transform_type(variable=params, var_type="dict", default={})
        page, is_pass = force_transform_type(variable=params.pop('page', 1), var_type="int", default=1)
        size, is_pass = force_transform_type(variable=params.pop('size', 10), var_type="int", default=10)
        user_group_id, is_pass = force_transform_type(variable=params.detail('user_group_id'), var_type="int")
        user_role_id, is_pass = force_transform_type(variable=params.detail('user_role_id'), var_type="int")

        # 查询排序字段处理
        sort = params.pop('sort', "-register_time")
        sort = sort if sort in ["register_time", "-register_time", "user_id", "-user_id"] else "-id"

        # 扩展字段映射字典
        field_map_list = DetailInfoService.field_map_list
        field_map = {item['field_index']: item['field'] for item in field_map_list}
        reversal_filed_map = {item['field']: item['field_index'] for item in field_map_list}

        # 默认返回字段去掉与的敏感信息
        # default_field_list = DetailInfoService.detail_fields + list(field_map.values()) + [
        #     "is_delete", "username", "fullname", "nickname", "user_id", "phone",
        #     "is_using", "user_identity", "subcompany_name"
        # ]
        # 默认返回字段去掉与的敏感信息
        default_field_list = DetailInfoService.detail_fields + list(field_map.values()) + [
            "is_delete", "username", "fullname", "nickname", "user_id", "phone",
            "is_using", "user_identity"
        ]
        # all_field_list = DetailInfoService.detail_fields + DetailInfoService.base_fields + list( field_map.values()) + ["subcompany_name"]
        all_field_list = DetailInfoService.detail_fields + DetailInfoService.base_fields + list(field_map.values())
        # 过滤字段处理
        filter_fields = format_list_handle(
            param_list=filter_fields_handler(
                input_field_expression=filter_fields,
                default_field_list=default_field_list,
            ),
            filter_filed_list=all_field_list,
            alias_dict=reversal_filed_map
        )
        # 查询ORM构建
        # subcompany_name_subquery = UserSubCompany.objects.filter(id=OuterRef('subcompany_id')).values('name')
        detail_info_obj = DetailInfo.objects.extra(
            select={"register_time": 'DATE_FORMAT(user_base_info.register_time, "%%Y-%%m-%%d %%H:%%i:%%s")'}
        ).filter(user__is_delete=0).annotate(
            username=F("user__username"),
            fullname=F("user__fullname"),
            nickname=F("user__nickname"),
            phone=F("user__phone"),
            email=F("user__email"),
            user_info=F("user__user_info"),
            privacies=F("user__privacies"),
            user_type=F("user__user_type"),
            user_identity=F("user__user_identity"),
            is_delete=F("user__is_delete"),
            is_using=F("user__is_using"),
            # subcompany_name=Subquery(subcompany_name_subquery),
        ).order_by(sort).values(*filter_fields)
        total = 0

        # id搜索,并且不分页，作为服务提供者
        # 搜索字典过滤处理,过滤掉不允许搜索的字段
        if not user_id_list is None and isinstance(user_id_list, list):
            # id列表参数校验，确保每一个参数都是有效得到int类型
            validate_user_id_list = []
            for i in user_id_list:
                var, valid = force_transform_type(variable=i, var_type="int")
                if valid:
                    continue
                validate_user_id_list.append(var)
            validate_user_id_list = list(set(validate_user_id_list))

            # 进行参数搜索
            res_list = detail_info_obj.filter(user_id__in=validate_user_id_list)

        elif user_role_id:
            # TODO 已经用了DJango的ORM，就不要用第三方的ORM了 20241206 by Sieyoo
            res_obj = {}
            # res_obj = db.table("role_user_to_role"). \
            #     select("role_user_to_role.role_id",
            #            "role_user_to_role.user_id",
            #            "uuid",
            #            "username",
            #            "fullname",
            #            "nickname",
            #            "phone",
            #            "email",
            #            "user_info",
            #            "user_type",
            #            "privacies",
            #            "register_time",
            #            "is_delete",
            #            "real_name",
            #            "sex",
            #            "birth",
            #            "tags",
            #            "signature",
            #            "avatar",
            #            "cover",
            #            "language",
            #            "region_code",
            #            "more",
            #            "field_1",
            #            "field_2",
            #            "field_3",
            #            "field_4",
            #            "field_5",
            #            "field_6",
            #            "field_7",
            #            "field_8",
            #            "field_9",
            #            "field_10",
            #            "field_11",
            #            "field_12",
            #            "field_13",
            #            "field_14",
            #            "field_15",
            #            ). \
            #     left_join("user_detail_info", "user_detail_info.user_id", "=", "role_user_to_role.user_id"). \
            #     left_join("user_base_info", "user_base_info.id", "=", "role_user_to_role.user_id"). \
            #     where("role_user_to_role.role_id", user_role_id)

            result_list = res_obj.paginate(size, page)
            total = result_list.total
            res_data = filter_result_field(
                result_list=filter_result_field(  # 扩展字段替换
                    result_list=result_list,
                    alias_dict=field_map,
                ),
                remove_filed_list=DetailInfoService.user_detail_remove_fields,  # 移除未配置的扩展字段已经主键ID
                alias_dict={"user": "user_id", "cover": "user_cover", "region_code": "user_region_code"}
                # 外键字段还原成ID，并起别名
            )

            return {'size': int(size), 'page': int(page), 'total': total, 'list': res_data}, None
        elif user_group_id:
            # TODO 已经用了DJango的ORM，就不要用第三方的ORM了 20241206 by Sieyoo
            res_obj = {}
            # res_obj = db.table("role_user_to_group"). \
            #     select("role_user_to_group.user_group_id",
            #            "role_user_to_group.user_id",
            #            "uuid",
            #            "username",
            #            "fullname",
            #            "nickname",
            #            "phone",
            #            "email",
            #            "user_info",
            #            "user_type",
            #            "privacies",
            #            "register_time",
            #            "is_delete",
            #            "real_name",
            #            "sex",
            #            "birth",
            #            "tags",
            #            "signature",
            #            "avatar",
            #            "cover",
            #            "language",
            #            "region_code",
            #            "more",
            #            "field_1",
            #            "field_2",
            #            "field_3",
            #            "field_4",
            #            "field_5",
            #            "field_6",
            #            "field_7",
            #            "field_8",
            #            "field_9",
            #            "field_10",
            #            "field_11",
            #            "field_12",
            #            "field_13",
            #            "field_14",
            #            "field_15",
            #            ). \
            #     left_join("user_detail_info", "user_detail_info.user_id", "=", "role_user_to_group.user_id"). \
            #     left_join("user_base_info", "user_base_info.id", "=", "role_user_to_group.user_id"). \
            #     where("role_user_to_group.user_group_id", user_group_id)

            result_list = res_obj.paginate(size, page)
            total = result_list.total
            res_data = filter_result_field(
                result_list=filter_result_field(  # 扩展字段替换
                    result_list=result_list,
                    alias_dict=field_map,
                ),
                remove_filed_list=DetailInfoService.user_detail_remove_fields,  # 移除未配置的扩展字段已经主键ID
                alias_dict={"user": "user_id", "cover": "user_cover", "region_code": "user_region_code"}
                # 外键字段还原成ID，并起别名
            )

            return {'size': int(size), 'page': int(page), 'total': total, 'list': res_data}, None
        else:
            search_params = format_params_handle(
                param_dict=params,
                filter_filed_list=all_field_list + ["register_time_start", "register_time_end", "user_id_list"],
                alias_dict=reversal_filed_map
            )
            search_params = format_params_handle(
                param_dict=search_params,
                alias_dict={
                    "user_type": "user_type__contains",
                    "username": "username__contains",
                    "nickname": "nickname__contains",
                    "email": "email__contains",
                    "phone": "phone__contains",
                    "fullname": "fullname__contains",
                    "real_name": "real_name__contains",
                    "register_time_start": "user__register_time__gte",
                    "register_time_end": "user__register_time__lte",
                    "user_id_list": "user__id__in"
                }
            )
            list_set = detail_info_obj.filter(**search_params)
            total = list_set.count()
            paginator = Paginator(list_set, size)
            try:
                list_set = paginator.page(page)
            except EmptyPage:
                list_set = paginator.page(paginator.num_pages)
            except Exception as e:
                return None, e.__str__()
            res_list = list(list_set.object_list)

        res_data = filter_result_field(
            result_list=filter_result_field(  # 扩展字段替换
                result_list=res_list,
                alias_dict=field_map,
            ),
            remove_filed_list=DetailInfoService.user_detail_remove_fields,  # 移除未配置的扩展字段已经主键ID
            alias_dict={"user": "user_id", "cover": "user_cover", "region_code": "user_region_code"}  # 外键字段还原成ID，并起别名
        )
        # 分情况返回数据
        if not user_id_list is None and isinstance(user_id_list, list):
            return res_data, None
        else:
            return {'size': int(size), 'page': int(page), 'total': total, 'list': res_data}, None

    @classmethod
    def get_detail(cls, user_id: int = None, user_uuid: str = None):
        """
        获取当前用户的基础信息和详细信息集合
        :param user_id: 通过用户ID搜索
        :param user_uuid: 通过用户UUID搜索
        :return: detail_info,err_msg
        """

        # 边界防传字串
        user_id, is_pass = JTransformType.to(user_id, "int", {})

        # 二选一必填
        if not user_id and not user_uuid:
            return None, '用户ID或UUID必填'

        # 查询用户信息
        user_set = BaseInfo.objects.annotate(user_id=F('id'), user_uuid=F('uuid'))
        if user_id:
            user_set = user_set.filter(id=user_id)
        elif user_uuid:
            user_set = user_set.filter(uuid=user_uuid)

        # 左连详细信息
        user_set = user_set.select_related('user_detail_info')  # select_related 用于正向查询，当左连表不存在时不会处理空值。

        # 日期格式化
        user_set = user_set.extra(select={
            'register_time': 'DATE_FORMAT(register_time, "%%Y-%%m-%%d %%H:%%i:%%s")',
            'birth_format': 'DATE_FORMAT(birth, "%%Y-%%m-%%d %%H:%%i:%%s")',  # 临时存放字段。因为左连表字段冲突
        })

        # 访问数据库
        user_set = user_set.first()
        if not user_set:
            return None, '用户不存在'
        detail_set = user_set.user_detail_info if hasattr(user_set, 'user_detail_info') else {}
        # print(f">detail_set:{detail_set}")

        # 获取拓展配置
        field_map_list = ExtendField.objects.all().values("field", 'field_index')

        # 生成字典数据
        whole_dict = {}
        base_dict = {k: getattr(user_set, k) for k in cls.base_fields}
        detail_dict = {k: getattr(detail_set, k) if hasattr(detail_set, k) else None for k in cls.detail_fields}
        extend_dict = {it['field']: getattr(detail_set, it['field_index']) for it in field_map_list if
                       hasattr(detail_set, it['field_index'])}
        whole_dict.update(base_dict, **detail_dict, **extend_dict)
        whole_dict['birth'] = user_set.birth_format  # 修正日期格式化时字段冲突失效的问题
        # print(f"whole_dict: {whole_dict}")

        # ================== section 追加信息 start ==============
        # # 获取用户审核信息
        # result_dict = ["UN_APPLY", "VERIFYING", "VERIFY_PASS", "VERIFY_REJECT"]
        # # print('> get_detail filter_dict:', filter_dict)
        # real_name_result = result_dict[filter_dict.get('real_name_is_pass', 0)]  # 实名认证状态
        # audit_result = result_dict[filter_dict.get('audit_status', 0)]  # 镖师认证状态
        # # print("> get_detail real_name_result, audit_result:", real_name_result, audit_result)
        #
        # if real_name_result == "VERIFY_REJECT":  # 审核不通过才需要原因
        #     reply_record = UserApplyRecord.objects.filter(user_id=user_id, apply_type_id=1,
        #                                                   result=real_name_result).order_by("-id").values(
        #         "reject_reason").first()
        #     if reply_record:
        #         filter_dict["reject_real_name_reason"] = reply_record["reject_reason"]
        # if audit_result == "VERIFY_REJECT":  #
        #     reply_record = UserApplyRecord.objects.filter(user_id=user_id, apply_type_id=2,
        #                                                   result=audit_result).order_by("-id").values(
        #         "reject_reason").first()
        #     if reply_record:
        #         filter_dict["reject_audit_status_reason"] = reply_record["reject_reason"]
        # # ================== section 追加信息 end ================

        return whole_dict, None

    @classmethod
    def create_or_update_detail(cls, params: dict = None, **kwargs):
        """
        添加或者更新用户的详细信息
        :param params: 添加/修改参数
        :return: None,err_msg
        """
        # 参数判断
        kwargs, is_pass = JTransformType.to(variable=kwargs, var_type="dict", default={})
        params, is_pass = JTransformType.to(variable=params, var_type="dict", default={})
        params.update(kwargs)

        user_id, is_pass = JTransformType.to(variable=params.pop('user_id', None), var_type="int")
        if not user_id:
            return None, "参数错误,user_id不能为空"
        # 判断用户是否存在
        user_base = BaseInfo.objects.filter(id=user_id)
        user_base_info = user_base.first()
        if not user_base_info:
            return None, '用户不存在'
        # =========== section 获取扩展字段的映射，默认值 start =================
        extend_field_list = ExtendField.objects.all().values("field", 'field_index', 'default')
        alias_dict = {item['field']: item['field_index'] for item in extend_field_list}  # 字段还原映射字典
        default_map = {item['field_index']: item['default'] for item in extend_field_list if
                       not item['default'] is None}  # 默认字段
        filter_filed_list = [i.name for i in DetailInfo._meta.fields]  # 字段列表
        # 强制类型转换,防止修改报错
        filter_filed_list.remove("id")
        filter_filed_list.remove("birth")
        filter_filed_list.remove("region_code")
        filter_filed_list.remove("more")
        # filter_filed_list.append("birth|date")
        filter_filed_list.append("region_code|int")
        filter_filed_list.append("more|dict")
        filter_filed_list.append("subcompany_id|int")
        # =========== section 获取扩展字段的映射，默认值 end   =================

        # =========== section 把扩展字段还原成 start =================
        # 剔除掉不是配置的扩展字段,还有原表的字段
        transformed_params = format_params_handle(
            param_dict=format_params_handle(
                param_dict=params,
                alias_dict=alias_dict
            ),
            filter_filed_list=filter_filed_list,
        )
        transformed_params.setdefault("user_id", user_id)
        # print("transformed_params >>>", transformed_params)
        # =========== section 把扩展字段还原成 end   =================
        # =========== section 进行数据库操作 start ============
        try:
            # 判断是否添加过
            detail_user_obj = DetailInfo.objects.filter(user_id=user_id)
            if not detail_user_obj.first():
                # 没有添加，进行添加操作
                transformed_params.pop("id", None)  # 添加的时候不能有ID主键，防止主键冲突
                # 在添加的时候给字段默认值
                for field_index, default in default_map.items():
                    transformed_params.setdefault(field_index, default)

                DetailInfo.objects.create(**transformed_params)
            else:
                # 添加过进行跟新
                detail_user_obj.update(**transformed_params)
            return None, None
        except Exception as e:
            return None, "参数配置错误：" + str(e)
        # =========== section 进行数据库操作 end   ============

    @classmethod
    def get_extend_fields(cls) -> list:
        """
        获取用户表扩展字段，前端渲染使用
        :return: fields_list,None
        """
        extend_fields = ExtendField.objects.order_by("-sort").all().values()
        return list(extend_fields), None

    @classmethod
    def set_extend_fields(cls, params) -> list:
        """
        获取用户表扩展字段，前端渲染使用。
        :param params 添加的参数
        :return: fields_list,None
        """
        allow_params = {}
        extend_fields = ExtendField.objects.order_by("-sort").all().values()
        return list(extend_fields), None



# -*- coding: utf-8 -*-
#
# Copyright 2017 Gehirn Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
from typing import (
    AbstractSet,
    Dict,
    Optional,
    Tuple,
)

from .exceptions import (
    JWSEncodeError,
    JWSDecodeError,
)
from .jwa import (
    supported_signing_algorithms,
    AbstractSigningAlgorithm,
)
from .jwk import AbstractJWKBase
from .utils import (
    b64encode,
    b64decode,
)

__all__ = ['JWS']


class JWS:

    def __init__(self) -> None:
        self._supported_algs = supported_signing_algorithms()

    def _retrieve_alg(self, alg: str) -> AbstractSigningAlgorithm:
        try:
            return self._supported_algs[alg]
        except KeyError:
            raise JWSDecodeError('Unsupported signing algorithm.')

    def encode(self, message: bytes, key: Optional[AbstractJWKBase] = None,
               alg='HS256',
               optional_headers: Optional[Dict[str, str]] = None) -> str:
        if alg not in self._supported_algs:  # pragma: no cover
            raise JWSEncodeError('unsupported algorithm: {}'.format(alg))
        alg_impl = self._retrieve_alg(alg)

        header = optional_headers.copy() if optional_headers else {}
        header['alg'] = alg

        header_b64 = b64encode(
            json.dumps(header, separators=(',', ':')).encode('ascii'))
        message_b64 = b64encode(message)
        signing_message = header_b64 + '.' + message_b64

        signature = alg_impl.sign(signing_message.encode('ascii'), key)
        signature_b64 = b64encode(signature)

        return signing_message + '.' + signature_b64

    def _decode_segments(
            self, message: str) -> Tuple[Dict[str, str], bytes, bytes, str]:
        try:
            signing_message, signature_b64 = message.rsplit('.', 1)
            header_b64, message_b64 = signing_message.split('.')
        except ValueError:
            raise JWSDecodeError('malformed JWS payload')

        header = json.loads(b64decode(header_b64).decode('ascii'))
        message_bin = b64decode(message_b64)
        signature = b64decode(signature_b64)
        return header, message_bin, signature, signing_message

    def decode(self, message: str, key: Optional[AbstractJWKBase] = None,
               do_verify=True,
               algorithms: Optional[AbstractSet[str]] = None) -> bytes:
        if algorithms is None:
            algorithms = set(supported_signing_algorithms().keys())

        header, message_bin, signature, signing_message = \
            self._decode_segments(message)

        alg_value = header['alg']
        if alg_value not in algorithms:
            raise JWSDecodeError('Unsupported signing algorithm.')

        alg_impl = self._retrieve_alg(alg_value)
        if do_verify and not alg_impl.verify(
                signing_message.encode('ascii'), key, signature):
            raise JWSDecodeError('JWS passed could not be validated')

        return message_bin



from django.db.models import Q
from ..models import ExtendField
from ..utils.custom_tool import format_params_handle

class ExtendFieldService:
    # 显式列出 ExtendField 模型的字段
    base_fields = [
        "id", "field_index", "field", "label", "type", "unit", "default", "config", "sort", "description"
    ]

    @classmethod
    def extend_field_list(cls, params=None):
        """
        扩展字段列表服务
        :param params: 搜索字典参数 dict
        :return: data, err
        """
        if params is None:
            params = {}

        # 处理搜索参数
        search_params = format_params_handle(
            param_dict=params,
            filter_filed_list=["field_index", "field", "label", "type", "unit", "default", "config", "sort", "description"],
            alias_dict={
                "field_index": "field_index__contains",
                "field": "field__contains",
                "label": "label__contains",
                "type": "type__contains",
                "unit": "unit__contains",
                "default": "default__contains",
                "config": "config__contains",
                "sort": "sort__contains",
                "description": "description__contains"
            },
            is_remove_empty=True,
        )

        # 获取未删除的扩展字段
        extend_field_set = ExtendField.objects.filter()

        # 根据搜索参数进行过滤
        if search_params:
            search_filters = [Q(**{key: value}) for key, value in search_params.items()]
            combined_filter = search_filters[0]
            for filter in search_filters[1:]:
                combined_filter |= filter
            extend_field_set = extend_field_set.filter(combined_filter)

        # 获取基础信息列表
        base_list = [{k: getattr(extend_field, k) for k in cls.base_fields} for extend_field in extend_field_set]

        return {'list': base_list}, None

    @classmethod
    def extend_field_add(cls, params):
        """
        添加扩展字段
        :param params: 添加参数
        :return: data, err
        """
        try:
            # 检查 field_index 是否已经存在
            if ExtendField.objects.filter(field_index=params.get("field_index")).exists():
                return None, "field_index 已存在"

            extend_field = ExtendField.objects.create(**params)
            return extend_field.to_json(), None
        except Exception as e:
            return None, str(e)

    @classmethod
    def extend_field_edit(cls, params, extend_field_id):
        """
        修改扩展字段
        :param params: 修改参数
        :param extend_field_id: 扩展字段ID
        :return: data, err
        """
        try:
            extend_field = ExtendField.objects.filter(id=extend_field_id).first()
            if not extend_field:
                return None, "扩展字段不存在"

            for key, value in params.items():
                setattr(extend_field, key, value)
            extend_field.save()

            return extend_field.to_json(), None
        except Exception as e:
            return None, str(e)

    @classmethod
    def extend_field_delete(cls, extend_field_id):
        """
        删除扩展字段
        :param extend_field_id: 扩展字段ID
        :return: data, err
        """
        try:
            extend_field = ExtendField.objects.filter(id=extend_field_id).first()
            if not extend_field:
                return None, "扩展字段不存在"

            extend_field.delete()
            return {"id": extend_field_id}, None
        except Exception as e:
            return None, str(e)


# _*_coding:utf-8_*_

class JDict(dict):
    def __init__(self, *args, **kwargs):
        dict.__init__(self, *args, **kwargs)
        self.__dict__ = self

    # 是否允许点操作符
    def allow_dotting(self, state=True):
        if state:
            self.__dict__ = self
        else:
            self.__dict__ = dict()

    # 当值不存在时返回None
    def __getattr__(self, *args):
        pass



# encoding: utf-8
"""
@project: djangoModel->test_login
@author: 孙楷炎
@Email: sky4834@163.com
@synopsis: 自带的单元测试
@created_time: 2023/4/18 22:06
"""

from django.test import TestCase

from ..services.user_service import UserService
from ..services.login_service import LoginService


class TestLogin(TestCase):
    __right = {
        "username": 'test',
        "password": '123456',
        "platform": '',
    }

    def setUp(self):
        pass

    def test_can_register(self):
        result, err = UserService.check_account(self.__right['username'], is_registered=False)
        print("> TestLogin: test_can_register:", result, err)
        self.assertEqual(err, None)

    def test_register(self):
        result, err = LoginService.register(*self.__right)
        print("> TestLogin: test_register:", result, err)
        self.assertEqual(err, None)

    # def test_login(self):
    #     """正确测试"""
    #     account_serv, error_text = UserService.check_account(self.username)
    #     assert error_text is None
    #     user_id = account_serv['user_id']
    #     auth_serv, error_text = UserService.check_login(
    #         user_id=user_id,
    #         password=self.password,
    #         account=self.username,
    #         platform=self.platform
    #     )

    #     assert error_text is None
    #
    # def test_with_out_username(self):
    #     """错误操作测试"""
    #     self.username = ""
    #     account_serv, error_text = UserService.check_account(self.username)
    #     assert error_text is None
    #     user_id = account_serv['user_id']
    #     auth_serv, error_text = UserService.check_login(
    #         user_id=user_id,
    #         password=self.password,
    #         account=self.username,
    #         platform=self.platform
    #     )
    #     assert error_text is None



from django.test import TestCase
from ..services.user_service import UserService
from ..utils.j_ulid_field import JULIDField
import datetime
from ulid import ULID


class TestUser(TestCase):
    def setUp(self):
        # 初始化测试数据
        self.test_params = {
            "account": "testuser",
            "password": "testpassword",
            "fullname": "Test User",
            "nickname": "testnick",
            "user_type": "PERSON",
            "phone": "1234567890",
            "email": "23944@qq.com"
        }

    __right = {
        "account": 'u001',
    }

    def test_add(self):
        result, err = UserService.user_add(self.__right)
        print("> UserBaseTest: test_add:", result, err)
        self.assertEqual(err, None)

    def test_detail(self):
        UserService.user_add(self.__right)
        exist, err = UserService.check_account('u001')
        print("> UserBaseTest: test_detail:", exist, err)
        self.assertEqual(err, None)

    def test_edit(self):
        UserService.user_add(self.__right)
        exist, err = UserService.check_account(self.__right['account'])
        print("> UserBaseTest: test_edit: 1 exist:", exist)
        exist['fullname'] = '路乙'
        result, err = UserService.user_edit(exist, user_id=exist['user_id'])
        print("> UserBaseTest: test_edit: 2 result:", result)
        self.assertEqual(err, None)

    def test_delete(self):
        UserService.user_add(self.__right)
        exist, err = UserService.check_account(self.__right['account'])
        print("> UserBaseTest: test_delete: 1 exist:", exist)

        result, err = UserService.user_delete(exist['user_id'])
        print("> ThreadClassifyTest::test_delete:", result)
        self.assertEqual(err, None)

    def test_uuid_delete(self):
        UserService.user_add(self.__right)
        exist, err = UserService.check_account(self.__right['account'])

        print("> UserBaseTest: test_delete: 1 exist:", exist)

        result, err = UserService.user_delete(user_uuid=exist['user_uuid'])
        print("> ThreadClassifyTest::test_delete:", result)
        self.assertEqual(err, None)